unit PointInsideContourUnit;

interface

uses ANEPIE, SysUtils, Classes;

function GContourArea(ListIndex : integer) : double;

function GPointInsideContour(ListToUse : ANE_INT32; X, Y : ANE_DOUBLE) : boolean;

procedure GPointInsideContourMMFun (const refPtX : ANE_DOUBLE_PTR      ;
				const refPtY : ANE_DOUBLE_PTR     ;
				numParams : ANE_INT16          ;
				const parameters : ANE_PTR_PTR ;
				myHandle : ANE_PTR            ;
				reply : ANE_PTR		       	); cdecl;

function GContourIntersectCell(ListToUse, Column, Row : ANE_INT32) : double;

implementation

uses BlockListUnit, CellVertexUnit, SegmentUnit, VertexUnit;

function ContourArea(AVertexList : TList) : double;
var
  VertexIndex : integer;
  AVertex, AnotherVertex : TBL_Vertex;
begin
  result := 0;
  if (AVertexList.Count < 3) then
  begin
    Exit;
  end;
  AVertex := AVertexList.Items[0];
  AnotherVertex := AVertexList.Items[AVertexList.Count -1];
  if (AVertex.XPos <> AnotherVertex.XPos) or
    (AVertex.YPos <> AnotherVertex.YPos) then
  begin
    Exit;
  end;

  For VertexIndex := 0 to AVertexList.Count -2 do
  begin
    AVertex := AVertexList.Items[VertexIndex];
    AnotherVertex := AVertexList.Items[VertexIndex+1];
    result := result + AVertex.XPos*AnotherVertex.YPos
      - AVertex.YPos*AnotherVertex.XPos;
  end;
  result := Abs(result/2);
end;

function PointInside(X, Y  : ANE_DOUBLE; AVertexList : TList) : boolean;
var
  VertexIndex : integer;
  AVertex, AnotherVertex : TBL_Vertex;
begin   // based on CACM 112
  AVertex := AVertexList.Items[0];
  AnotherVertex := AVertexList.Items[AVertexList.Count -1];
  if (AVertex.XPos <> AnotherVertex.XPos) or
    (AVertex.YPos <> AnotherVertex.YPos) then
  begin
    result := False;
    Exit;
  end;

  result := true;
  For VertexIndex := 0 to AVertexList.Count -2 do
  begin
    AVertex := AVertexList.Items[VertexIndex];
    AnotherVertex := AVertexList.Items[VertexIndex+1];
    if ((Y <= AVertex.YPos) = (Y > AnotherVertex.YPos)) and
       (X - AVertex.XPos - (Y - AVertex.YPos) *
         (AnotherVertex.XPos - AVertex.XPos)/
         (AnotherVertex.YPos - AVertex.YPos) < 0) then
      begin
        result := not result;
      end;
  end;
  result := not result;
end;

function ContourArea2(ListIndex : integer) : double;
var
  Index, InnerIndex : integer;
  AVertexList, AnotherVertexList, ThirdVertexList : TList;
  InnerList : TList;
  Vertex1, Vertex2 : TBL_Vertex;
  AddList : Boolean;
begin
  AVertexList := MainVertexList.Items[ListIndex];
  result := ContourArea(AVertexList);
  InnerList := TList.Create;
  try
    for Index := 0 to MainVertexList.Count -1 do
    begin
      if Index <> ListIndex then
      begin
        AnotherVertexList := MainVertexList[Index];
        Vertex1 := AnotherVertexList[0];
        if PointInside(Vertex1.XPos, Vertex1.YPos, AVertexList) then
        begin
          AddList := True;
          for InnerIndex := InnerList.Count-1 downto 0 do
          begin
            ThirdVertexList := InnerList[InnerIndex];
            if PointInside(Vertex1.XPos, Vertex1.YPos, ThirdVertexList) then
            begin
              AddList := False;
              break;
            end;
            Vertex2 := ThirdVertexList[0];
            if PointInside(Vertex2.XPos, Vertex2.YPos, AnotherVertexList) then
            begin
              InnerList.Delete(InnerIndex);
            end;
          end;
          if AddList then
          begin
            InnerList.Add(AnotherVertexList);
          end;
        end;
      end;
    end;

    for Index := 0 to InnerList.Count -1 do
    begin
      AnotherVertexList := InnerList[Index];
      result := result - ContourArea(AnotherVertexList);
    end;
  finally
    InnerList.Free;
  end;
end;

function GContourArea(ListIndex : integer) : double;
begin
  result := ContourArea2(ListIndex);
end;

function AddVerticiesOnEdge(AVertexList : TList; ASegment : TSegment): boolean;
var
  Index : integer;
  IntersectVertex : TVertex;
  EdgeSegment : TSegment;
  FirstVertex, SecondVertex : TVertex;
  V1, V2, V3 : TBL_Vertex;
begin
  result := False;
  for Index := AVertexList.Count -1 downto 1 do
  begin
    V1 := AVertexList[Index];
    V2 := AVertexList[Index-1];
    FirstVertex := TVertex.Create;
    SecondVertex := TVertex.Create;
    IntersectVertex := TVertex.Create;
    try
      FirstVertex.X := V1.XPos;
      FirstVertex.Y := V1.YPos;
      SecondVertex.X := V2.XPos;
      SecondVertex.Y := V2.YPos;
      EdgeSegment := TSegment.Create(FirstVertex,SecondVertex);
      try
        if ASegment.Intersection(EdgeSegment,IntersectVertex) then
        begin
          V3 := TBL_Vertex.Create;
          V3.XPos := IntersectVertex.X;
          V3.YPos := IntersectVertex.Y;
          AVertexList.Insert(Index,V3);
          result := True;
        end;
      finally
        EdgeSegment.Free;
      end;
    finally
      FirstVertex.Free;
      SecondVertex.Free;
      IntersectVertex.Free;
    end;
  end;
end;

function AddVerticiesOnEdges(AVertexList : TList;
  LeftEdge, RightEdge, TopEdge, BottomEdge : TReal) : boolean;
var
  TopLeft, TopRight, BottomLeft, BottomRight : TVertex;
  LeftSeg, RightSeg, TopSeg, BotSeg : TSegment;
begin
  TopLeft := TVertex.Create;
  TopRight := TVertex.Create;
  BottomLeft := TVertex.Create;
  BottomRight := TVertex.Create;
  try
    TopLeft.X := LeftEdge.Value;
    TopRight.X := RightEdge.Value;
    BottomLeft.X := LeftEdge.Value;
    BottomRight.X := RightEdge.Value;
    TopLeft.Y := TopEdge.Value;
    TopRight.Y := TopEdge.Value;
    BottomLeft.Y := BottomEdge.Value;
    BottomRight.Y := BottomEdge.Value;
    LeftSeg := TSegment.Create(TopLeft,BottomLeft);
    RightSeg := TSegment.Create(TopRight,BottomRight);
    TopSeg := TSegment.Create(TopLeft,TopRight);
    BotSeg := TSegment.Create(BottomRight,BottomLeft);
    try
      result := AddVerticiesOnEdge(AVertexList, LeftSeg);
      result := AddVerticiesOnEdge(AVertexList, RightSeg) or result;
      result := AddVerticiesOnEdge(AVertexList, TopSeg)   or result;
      result := AddVerticiesOnEdge(AVertexList, BotSeg)   or result;
    finally
      LeftSeg.Free;
      RightSeg.Free;
      TopSeg.Free;
      BotSeg.Free;
    end;

  finally
    TopLeft.Free;
    TopRight.Free;
    BottomLeft.Free;
    BottomRight.Free;
  end;

end;

function GetContourAreaInCell (Column, Row : ANE_INT32;
  AVertexList : TList): ANE_DOUBLE;
var
  LeftEdge, RightEdge, TopEdge, BottomEdge : TReal;
  VerticiesInCell : TList;
  Index : integer;
  AVertex : TBL_Vertex;
  Temp : TReal;
  CellCorners : TList;
begin
  RightEdge := ColumnList.Items[Column];
  LeftEdge := ColumnList.Items[Column-1];
  if LeftEdge.Value > RightEdge.Value then
  begin
    Temp := LeftEdge;
    LeftEdge := RightEdge;
    RightEdge := Temp;
  end;
  TopEdge := RowList.Items[Row];
  BottomEdge := RowList.Items[Row-1];
  if BottomEdge.Value > TopEdge.Value then
  begin
    Temp := BottomEdge;
    BottomEdge := TopEdge;
    TopEdge := Temp;
  end;
  VerticiesInCell := TList.Create;
  try
    for Index := 0 to AVertexList.Count -1 do
    begin
      AVertex := AVertexList[Index];
      AVertex := AVertex.CopyOfVertex;
      VerticiesInCell.Add(AVertex);
    end;
    if not AddVerticiesOnEdges(VerticiesInCell,
      LeftEdge, RightEdge, TopEdge, BottomEdge) then
    begin
      if not PointInside(LeftEdge.Value, TopEdge.Value, VerticiesInCell) then
      begin
        CellCorners := TList.Create;
        try
          AVertex := TBL_Vertex.Create;
          AVertex.XPos := LeftEdge.Value;
          AVertex.YPos := TopEdge.Value;
          CellCorners.Add(AVertex);

          AVertex := TBL_Vertex.Create;
          AVertex.XPos := RightEdge.Value;
          AVertex.YPos := TopEdge.Value;
          CellCorners.Add(AVertex);

          AVertex := TBL_Vertex.Create;
          AVertex.XPos := RightEdge.Value;
          AVertex.YPos := BottomEdge.Value;
          CellCorners.Add(AVertex);

          AVertex := TBL_Vertex.Create;
          AVertex.XPos := LeftEdge.Value;
          AVertex.YPos := BottomEdge.Value;
          CellCorners.Add(AVertex);

          AVertex := TBL_Vertex.Create;
          AVertex.XPos := LeftEdge.Value;
          AVertex.YPos := TopEdge.Value;
          CellCorners.Add(AVertex);

          AVertex := VerticiesInCell[0];
          if not PointInside(AVertex.XPos, AVertex.YPos, CellCorners) then
          begin
            result := 0;
            Exit;
          end;
        finally
          for Index := 0 to CellCorners.Count -1 do
          begin
            TBL_Vertex(CellCorners[Index]).Free;
          end;

          CellCorners.Free;
        end;
      end;
    end;
    for Index := 0 to VerticiesInCell.Count -1 do
    begin
      AVertex := VerticiesInCell[Index];
      if AVertex.YPos > TopEdge.Value then
      begin
        AVertex.YPos := TopEdge.Value
      end;
      if AVertex.YPos < BottomEdge.Value then
      begin
        AVertex.YPos := BottomEdge.Value
      end;
      if AVertex.XPos > RightEdge.Value then
      begin
        AVertex.XPos := RightEdge.Value
      end;
      if AVertex.XPos < LeftEdge.Value then
      begin
        AVertex.XPos := LeftEdge.Value
      end;
    end;
    result := ContourArea(VerticiesInCell);
  finally
    for Index := VerticiesInCell.Count -1 downto 0 do
    begin
      AVertex := VerticiesInCell[Index];
      AVertex.Free;
    end;
    VerticiesInCell.Free;
  end;

end;

function GContourIntersectCell(ListToUse, Column, Row : ANE_INT32) : double;
var
  AVertexList, AnotherVertexList : TList;
  AVertex, AnotherVertex : TBL_Vertex;
  ListIndex : integer;
begin
  AVertexList := MainVertexList.Items[ListToUse];
  if AVertexList.Count < 3
  then
  begin
    result := 0;
  end  // if AVertexList.Count < 3
  else
  begin
    AVertex := AVertexList.Items[0];
    AnotherVertex := AVertexList.Items[AVertexList.Count-1];
    if (AVertex.XPos <> AnotherVertex.XPos)
      or (AVertex.YPos <> AnotherVertex.YPos) then
    begin
      // contour is an open contour.
      result := 0;
    end // if not ((AVertex.XPos = AnotherVertex.XPos) and (AVertex.YPos = AnotherVertex.YPos))
    else
    begin
      result := GetContourAreaInCell (Column, Row, AVertexList);
      if result > 0 then
      begin
        for ListIndex := 0 to MainVertexList.Count -1 do
        begin
          if ListIndex <> ListToUse then
          begin
            AnotherVertexList := MainVertexList[ListIndex];
            AVertex := AnotherVertexList[0];
            if PointInside(AVertex.XPos, AVertex.YPos , AVertexList) then
            begin
              result := result -
                GetContourAreaInCell (Column, Row, AnotherVertexList)
            end;
          end;
        end;
      end;
    end; // if not ((AVertex.XPos = AnotherVertex.XPos) and (AVertex.YPos = AnotherVertex.YPos)) else
  end;  // if AVertexList.Count < 3 else
end;

function GPointInsideContour(ListToUse : ANE_INT32; X, Y : ANE_DOUBLE) : boolean;
var
  AVertexList, AnotherVertexList : TList;
  AVertex, AnotherVertex : TBL_Vertex;
  ListIndex : integer;
begin
      AVertexList := MainVertexList.Items[ListToUse];
      if AVertexList.Count < 3
      then
        begin
          result := False;
        end  // if AVertexList.Count < 3
      else
        begin
          AVertex := AVertexList.Items[0];
          AnotherVertex := AVertexList.Items[AVertexList.Count-1];
          if not ((AVertex.XPos = AnotherVertex.XPos) and (AVertex.YPos = AnotherVertex.YPos))
          then
            begin
              // contour is an open contour.
              result := False;
            end // if not ((AVertex.XPos = AnotherVertex.XPos) and (AVertex.YPos = AnotherVertex.YPos))
          else
            begin
              result := PointInside(X, Y , AVertexList);
              if result then
              begin
                for ListIndex := 0 to MainVertexList.Count -1 do
                begin
                  if ListIndex <> ListToUse then
                  begin
                    AnotherVertexList := MainVertexList[ListIndex];
                    AVertex := AnotherVertexList[0];
                    if PointInside(AVertex.XPos, AVertex.YPos , AVertexList) then
                    begin
                      if PointInside(X, Y, AnotherVertexList) then
                      begin
                        result := False;
                        break;
                      end;
                    end;
                  end;
                end;
              end;


            end; // if not ((AVertex.XPos = AnotherVertex.XPos) and (AVertex.YPos = AnotherVertex.YPos)) else
        end;  // if AVertexList.Count < 3 else
end;

procedure GPointInsideContourMMFun (const refPtX : ANE_DOUBLE_PTR      ;
				const refPtY : ANE_DOUBLE_PTR     ;
				numParams : ANE_INT16          ;
				const parameters : ANE_PTR_PTR ;
				myHandle : ANE_PTR            ;
				reply : ANE_PTR		       	); cdecl;

var
  result : ANE_BOOL;
  param : PParameter_array;
  param1_ptr : ANE_INT32_PTR;
  ListToUse : ANE_INT32;
  param2_ptr : ANE_DOUBLE_PTR;
  X : ANE_DOUBLE;
  param3_ptr : ANE_DOUBLE_PTR;
  Y : ANE_DOUBLE;
begin
  try
    begin
      param := @parameters^;
      param1_ptr :=  param^[0];
      ListToUse :=  param1_ptr^;
      param2_ptr :=  param^[1];
      X :=  param2_ptr^;
      param3_ptr :=  param^[2];
      Y :=  param3_ptr^;
      result := GPointInsideContour(ListToUse, X, Y);
    end  // try
  except
    on Exception do
      begin
        Inc(ErrorCount);
        result := False;
      end;
  end;

  ANE_BOOL_PTR(reply)^ := result;
end;

end.
