unit EditUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Clipbrd,

// We need to use the appropriate units. In this example, we have an import
// PIE so we need to use ImportPIE.pas. All PIE's use AnePIE.
   AnePIE, ImportPIE, ExtCtrls, Buttons, ArgusDataEntry ;

type
  TPoint = class(TObject)
  private
    function GetXCoord : Integer;
    function GetYCoord : Integer;
    Procedure SetXCoord(AnX : Integer);
    Procedure SetYCoord(AY  : Integer);
  public
    X : Extended;
    Y : Extended;
    function PointToString : string;
    procedure StringToPoint(AString : string);
    Property XCoord : Integer read GetXCoord write SetXCoord;
    property YCoord : Integer read GetYCoord write SetYCoord;
    Procedure Draw;
    function Selected(AnX, AY : integer) : boolean;
  end;

  TfrmEditContours = class(TForm)
    Panel1: TPanel;
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    ScrollBox1: TScrollBox;
    PaintBox1: TPaintBox;
    btnZoomIn: TSpeedButton;
    btnZoomOut: TSpeedButton;
    Shape1: TShape;
    Label1: TLabel;
    Label2: TLabel;
    BitBtn1: TBitBtn;
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
    procedure PaintBox1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure btnZoomInClick(Sender: TObject);
    procedure btnZoomOutClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure PaintBox1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintBox1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    MinX, MinY, MaxX, MaxY: Extended;
    Multiplier, DefaultMultiplier : Extended;
    StartX, StartY, EndX, EndY : Integer;
    ZoomBegun : Boolean;
    { Private declarations }
  public
    ContourList : TList;
    CurrentPoint : TPoint;
    Procedure ReadContours(AString : String);
    procedure ReadData(ADataString: String);
    function WriteContours : string;
    procedure GetMinMax;
    function Select(AnX, AY : integer) : TPoint;
    procedure ReverseContours;
    { Public declarations }
  end;

  TContour = class(TObject)
  private
    FPoints : TList;
    PointsReady : boolean;
    function GetPoints(Index: integer): TPoint;
    procedure SetPoints(Index: integer; const Value: TPoint);
    procedure StringsToPoints;
    Procedure PointsToStrings;
    function Select(AnX, AY : integer) : TPoint;
  public
    Value : string;
    Heading : TStringlist;
    PointStrings : TStringList;
    constructor Create;
    Destructor Destroy; override;
    Property PointValues[Index : integer] : TPoint read GetPoints write SetPoints;
    Procedure Draw;
  end;


var
  frmEditContours: TfrmEditContours;

// You must use the cdecl calling convention for all functions that will be
// called by Argus ONE or calls back to Argus ONE.
procedure GetANEFunctions( var numNames : ANE_INT32;
          var descriptors : ANEPIEDescPtr ); cdecl;

procedure GEditContoursPIE(aneHandle : ANE_PTR; const  fileName : ANE_STR;
                               layerHandle : ANE_PTR); cdecl;

implementation



{$R *.DFM}

{We use ANECB in this case because we use the ANE_ImportTextToLayer procedure.}

uses ANECB, NodeUnit, UtilityFunctions;

//  kMaxFunDesc is the maximum number of PIE's in the dll
const kMaxFunDesc = 3;
      Margin = 20;

// global variables.
var
   gFunDesc : array [0..kMaxFunDesc-1] of ANEPIEDescPtr;   // list of PIE descriptors for all parts

   gEditContoursPIEDesc  : ANEPIEDesc;	                   // PIE descriptor
   gEditContoursPIEImportPIEDesc : ImportPIEDesc;                // ImportPIE descriptor

   gReverseContoursPIEDesc  : ANEPIEDesc;	                   // PIE descriptor
   gReverseContoursPIEImportPIEDesc : ImportPIEDesc;                // ImportPIE descriptor

   gData2ContoursPIEDesc  : ANEPIEDesc;	                   // PIE descriptor
   gData2ContoursPIEImportPIEDesc : ImportPIEDesc;                // ImportPIE descriptor


procedure TContour.StringsToPoints;
var
  index : Integer;
  AString : string;
  APoint : TPoint;
begin
  for Index := FPoints.Count -1 downto 0 do
  begin
    TPoint(FPoints[Index]).Free;
  end;
  FPoints.Clear;
  For Index := 0 to PointStrings.Count - 1 do
  begin
    AString := PointStrings[Index];
    if AString <> '' then
    begin
      APoint := TPoint.Create;
      APoint.StringToPoint(AString);
      FPoints.Add(APoint);
    end;
  end;
  PointsReady := True;

end;

constructor TContour.Create;
begin
    inherited;
    Heading := TStringlist.Create;
    PointStrings := TStringList.Create;
    FPoints := TList.Create;
    PointsReady := False;
end;

destructor TContour.Destroy;
var
    Index : integer;
    PointIndex : integer;
    APoint : TPoint;
begin
  Heading.Free;
  PointStrings.Free;
  for Index := FPoints.Count -1 downto 0 do
  begin
    APoint := FPoints[Index];
    if APoint <> nil then
    begin
      FPoints.Delete(Index);
      PointIndex := FPoints.IndexOf(APoint);
      if PointIndex > -1 then
      begin
        FPoints[PointIndex] := nil;
      end;
      APoint.Free;
    end;
  end;
  FPoints.Free;
  Inherited;
end;

procedure TfrmEditContours.ReadContours(AString : String);
var
  ContourStringList : TStringList;
  Index : integer;
  AContour : TContour;
  HeadingTest : boolean;
begin
  ContourStringList := TStringList.Create;
  ContourList := TList.Create;
  ContourStringList.Text := AString;
  AContour := nil;
  HeadingTest := False;
  For Index := 0 to ContourStringList.Count -1 do
  begin
    if ContourStringList.Strings[Index] = '' then
    begin
      HeadingTest := False;
    end;
    if not HeadingTest and (Pos('#', ContourStringList.Strings[Index]) > 0) then
    begin
      AContour := TContour.Create;
      ContourList.Add(AContour);
      HeadingTest := True;
    end;
    if not (AContour = nil) then
    begin
      if HeadingTest
      then
        begin
          AContour.Heading.Add(ContourStringList.Strings[Index]);
        end
      else
        begin
          if not (ContourStringList.Strings[Index] = '') then
          begin
            AContour.PointStrings.Add(ContourStringList.Strings[Index]);
          end;
        end;
      if Pos('# X pos', ContourStringList.Strings[Index]) > 0 then
      begin
        HeadingTest := False;
      end;
    end;
  end;
  For index := 0 to ContourList.Count -1 do
  begin
    AContour := ContourList.Items[index];
    if not AContour.PointsReady then
    begin
      AContour.StringsToPoints;
    end;
    if AContour.FPoints.Count > 1 then
    begin
      if (AContour.PointValues[0].X = AContour.PointValues[AContour.FPoints.Count-1].X) and
         (AContour.PointValues[0].y = AContour.PointValues[AContour.FPoints.Count-1].y) then
      begin
        TPoint(AContour.FPoints.Items[AContour.FPoints.Count-1]).Free;
        AContour.FPoints.Items[AContour.FPoints.Count-1] := AContour.FPoints.Items[0];
      end;
    end;
  end;

  ContourStringList.Free;

end;

procedure TfrmEditContours.ReadData(ADataString : String);
var
  ContourStringList : TStringList;
  Index : integer;
  AContour : TContour;
  Value : String;
  TabPosition : integer;
  ADataPointString : String;
  X, Y : double;
  APoint : TPoint;

begin
  ContourStringList := TStringList.Create;
  try
    ContourList := TList.Create;
    ContourStringList.Text := ADataString;
{    if ContourStringList.Count > 0 then
    begin
      Value := ContourStringList[0];
      TabPosition := Pos(Chr(9), Value);
      if TabPosition > 0 then
      begin
        Value := Copy(Value, TabPosition+1, Length(Value));
      end
      else
      begin
        Value := '0';
      end;
    end;  }
    For Index := 2 to ContourStringList.Count -1 do
    begin
      ADataPointString := ContourStringList[Index];
      if ADataPointString <> '' then
      begin
        AContour := TContour.Create;
        ContourList.Add(AContour);

        TabPosition := Pos(Chr(9), ADataPointString);
        X := InternationalStrToFloat(Copy(ADataPointString,1,TabPosition-1));
        ADataPointString := Copy(ADataPointString, TabPosition+1, Length(ADataPointString));

        TabPosition := Pos(Chr(9), ADataPointString);
        Y := InternationalStrToFloat(Copy(ADataPointString,1,TabPosition-1));
        ADataPointString := Copy(ADataPointString, TabPosition+1, Length(ADataPointString));

        AContour.Heading.Add('## Name:');
        AContour.Heading.Add('## Icon:0');
        AContour.Heading.Add('# Points Count' + Chr(9) + 'Value');
        AContour.Heading.Add('1' + Chr(9) + ADataPointString);
        AContour.Heading.Add('# X pos' + Chr(9) + 'Y pos');

        APoint := TPoint.Create;
        APoint.X := X;
        APoint.Y := Y;
        AContour.FPoints.Add(APoint);
      end;

    end;
  finally

  ContourStringList.Free;
  end;

end;

procedure GData2ContourPIE(aneHandle : ANE_PTR; const  fileName : ANE_STR;
          layerHandle : ANE_PTR); cdecl;
var
  DataText : ANE_STR;
  DataTextString : String;
  ImportText : string;
begin
  try
    begin
      ANE_ExportTextFromOtherLayer(aneHandle, layerHandle, @DataText );
      DataTextString := String(DataText);
      frmEditContours := TfrmEditContours.Create(Application);
      try
        frmEditContours.ReadData(DataTextString);
        ImportText := frmEditContours.WriteContours;

        ANE_ImportTextToLayer(aneHandle, PChar(ImportText));
      finally
        frmEditContours.Free;
      end;
    end;
   except
     on Exception do
        begin
        end;
   end;
end;

procedure GEditContoursPIE(aneHandle : ANE_PTR; const  fileName : ANE_STR;
          layerHandle : ANE_PTR); cdecl;
var
  InfoText : ANE_STR;
  InfoTextString : String;
  ImportText : string;
begin
  try
    begin
          ANE_ExportTextFromOtherLayer(aneHandle, layerHandle, @InfoText );
          InfoTextString := String(InfoText);

          frmEditContours := TfrmEditContours.Create(Application);
          frmEditContours.ReadContours(InfoTextString);
          frmEditContours.GetMinMax;
          frmEditContours.Multiplier := frmEditContours.DefaultMultiplier;
          if frmEditContours.ShowModal = mrOK then
          begin
            ImportText := frmEditContours.WriteContours;
            frmEditContours.Free;

            ANE_LayerClear(aneHandle , layerHandle, False );
            ANE_ImportTextToLayerByHandle(aneHandle, layerHandle, PChar(ImportText));
          end;

    end;
   except
     on Exception do
        begin
        end;
   end;
end;

procedure GReverseContoursPIE(aneHandle : ANE_PTR; const  fileName : ANE_STR;
          layerHandle : ANE_PTR); cdecl;
var
//  InfoText : ANE_STR;
  InfoTextString : String;
  ImportText : string;
//  AClipBoard : TClipboard;
begin
  try
    begin
//          ANE_ExportTextFromOtherLayer(aneHandle, layerHandle, @InfoText );
//          InfoTextString := String(InfoText);

          InfoTextString := Clipboard.AsText;

          frmEditContours := TfrmEditContours.Create(Application);
          frmEditContours.ReadContours(InfoTextString);
          frmEditContours.ReverseContours;
{          frmEditContours.GetMinMax;
          frmEditContours.Multiplier := frmEditContours.DefaultMultiplier;
          if frmEditContours.ShowModal = mrOK then
          begin    }
            ImportText := frmEditContours.WriteContours;
            frmEditContours.Free;

//            ANE_LayerClear(aneHandle , layerHandle, False );
            ANE_ImportTextToLayerByHandle(aneHandle, layerHandle, PChar(ImportText));
//          end;

    end;
   except
     on Exception do
        begin
        end;
   end;
end;


function TContour.GetPoints(Index: integer): TPoint;
begin
  if not PointsReady then
  begin
    StringsToPoints;
  end;
  result := FPoints.Items[Index];
end;

procedure TContour.SetPoints(Index: integer; const Value: TPoint);
begin
  FPoints.Items[Index] := Value;
end;

{ TPoint }

procedure TPoint.Draw;
const
  size = 2;
var
  AColor : TColor;
begin
  AColor := frmEditContours.PaintBox1.Canvas.Brush.Color;
  if self <> frmEditContours.CurrentPoint then
  begin
    frmEditContours.PaintBox1.Canvas.Brush.Color := clBlack;
  end
  else
  begin
    frmEditContours.PaintBox1.Canvas.Brush.Color := clWhite;
  end;
  frmEditContours.PaintBox1.Canvas.Rectangle(XCoord-size,YCoord-size,XCoord+size,YCoord+size);
  frmEditContours.PaintBox1.Canvas.Brush.Color := AColor;
end;

function TPoint.GetXCoord: Integer;
begin
  result := Round((X*frmEditContours.Multiplier
    - frmEditContours.MinX*frmEditContours.Multiplier) + Margin);
end;

function TPoint.GetYCoord: Integer;
begin
  result := frmEditContours.PaintBox1.Height -
    Round((Y*frmEditContours.Multiplier
      - frmEditContours.MinY*frmEditContours.Multiplier) + Margin);

end;

function TPoint.PointToString: string;
var
  CommaPosition : integer;
begin
  result := FloatToStr(x) + Chr(9) + FloatToStr(y);
  CommaPosition := Pos(',',result);
  While CommaPosition > 0 do
  begin
    result[CommaPosition] := '.';
    CommaPosition := Pos(',',result);
  end;
end;


function TPoint.Selected(AnX, AY: integer): boolean;
var
  XPosition , YPosition : integer;
begin
  XPosition := XCoord;
  YPosition := YCoord;
  if (AnX < XPosition + 3) and (AnX > XPosition - 3) and
    (AY < YPosition + 3) and (AY > YPosition - 3) then
  begin
    result := True;
  end
  else
  begin
    result := False;
  end;
end;

procedure TPoint.SetXCoord(AnX: Integer);
begin
  X :=  (AnX -  Margin)/frmEditContours.Multiplier + frmEditContours.MinX
end;

procedure TPoint.SetYCoord(AY: Integer);
begin
   Y := (frmEditContours.PaintBox1.Height - AY -
     Margin )/frmEditContours.Multiplier + frmEditContours.MinY ;

end;

procedure TPoint.StringToPoint(AString: string);
var
  XString, YString : string ;
  TabPosition : integer;
begin
    TabPosition := Pos(Chr(9),AString);
    XString := copy(AString, 1, TabPosition-1);
    YString := Copy(AString, TabPosition+1, Length(AString));
    X := InternationalStrToFloat(XString);
    Y := InternationalStrToFloat(YString);

end;

procedure TContour.PointsToStrings;
var
    Index : integer;
begin
  PointStrings.Clear;
//  PointStrings.Add('');
  for Index := 0 to FPoints.Count -1 do
  begin
    PointStrings.Add(TPoint(FPoints[Index]).PointToString);
  end;

end;

procedure TfrmEditContours.FormDestroy(Sender: TObject);
var
  Index : integer;
  AContour : TContour;
begin
  for index := ContourList.Count -1 downto 0 do
  begin
    AContour := ContourList.Items[index];
    AContour.Free;
  end;
  ContourList.Free;

end;

procedure TfrmEditContours.FormCreate(Sender: TObject);
var
  DLLPath : string;
begin
  ContourList := TList.Create;
  GetDllDirectory(GetDLLName, DLLPath);
  HelpFile := DLLPath + '\EditContours.hlp';
end;

procedure TContour.Draw;
var
  APoint, FirstPoint : TPoint;
  var Index : integer;

begin
  if not PointsReady then
  begin
    StringsToPoints;
  end;
  if FPoints.Count > 0 then
  begin
    FirstPoint := PointValues[0];
    frmEditContours.PaintBox1.Canvas.MoveTo(FirstPoint.XCoord,FirstPoint.YCoord);
    FirstPoint.Draw;
    For Index := 1 to FPoints.Count -1 do
    begin
      APoint := PointValues[Index];
      frmEditContours.PaintBox1.Canvas.LineTo(APoint.XCoord,APoint.YCoord);
      APoint.Draw;
    end;
  end;
end;

procedure TfrmEditContours.PaintBox1Paint(Sender: TObject);
var
  Index : integer;
  AContour : TContour;
begin
  for Index := 0 to ContourList.Count -1 do
  begin
    AContour := ContourList[Index];
    AContour.Draw;
  end;
end;

procedure TfrmEditContours.GetMinMax;
  var
    ContourIndex, PointIndex : integer;
    AContour : TContour;
    FoundFirst : boolean;
    AValue : Extended;
    XMultiplier, YMultiplier : Extended;
begin
  FoundFirst := False;
  for ContourIndex := 0 to Contourlist.Count -1 do
  begin
    AContour := Contourlist[ContourIndex];
    if not AContour.PointsReady then
    begin
      AContour.StringsToPoints;
    end;
    for PointIndex := 0 to AContour.FPoints.Count -1 do
    begin
      MinX := AContour.PointValues[PointIndex].X;
      MinY := AContour.PointValues[PointIndex].Y;
      MaxX := MinX;
      MaxY := MinY;
      FoundFirst := True;
      break;
    end;
    if FoundFirst then
    begin
      break;
    end;
  end;
  if FoundFirst then
  begin
    for ContourIndex := 0 to Contourlist.Count -1 do
    begin
      AContour := Contourlist[ContourIndex];
      if not AContour.PointsReady then
      begin
        AContour.StringsToPoints;
      end;
      for PointIndex := 0 to AContour.FPoints.Count -1 do
      begin
        AValue := AContour.PointValues[PointIndex].X;
        if AValue < MinX then MinX := AValue;
        if AValue > MaxX then MaxX := AValue;
        AValue := AContour.PointValues[PointIndex].Y;
        if AValue < MinY then MinY := AValue;
        if AValue > MaxY then MaxY := AValue;
      end;
    end;
  end;
  if MaxX = MinX then
  begin
    MaxX := MaxX + 5;
  end;
  if MaxY = MinY then
  begin
    MaxY := MaxY + 5;
  end;
  XMultiplier := (Paintbox1.Width-2*Margin)/(MaxX-MinX);
  YMultiplier := (Paintbox1.Height-2*Margin)/(MaxY-MinY);
  if XMultiplier < YMultiplier
  then
    begin
      DefaultMultiplier := XMultiplier;
    end
  else
    begin
      DefaultMultiplier := YMultiplier;
    end;
end;

function TContour.Select(AnX, AY: integer): TPoint;
var
  Index : integer;
  APoint : TPoint;
begin
  result := nil;
  for Index := 0 to FPoints.Count -1 do
  begin
    APoint := FPoints[Index];
    if APoint.Selected(AnX, AY) then
    begin
      result := APoint;
      break;
    end;
  end;

end;

function TfrmEditContours.Select(AnX, AY: integer): TPoint;
var
  index : integer;
  AContour : TContour;
  APoint : TPoint;
begin
  result := nil;
  for Index := 0 to ContourList.Count -1 do
  begin
    AContour := ContourList[Index];
    APoint := AContour.Select(AnX, AY);
    if APoint <> nil then
    begin
      result := APoint;
    end;
  end;
end;

procedure TfrmEditContours.PaintBox1MouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  XMult, YMult, LocalMult : Extended;
  LeftX, RightX, TopY, BottomY : integer;
  APoint : TPoint;
  Width, Height : Integer;
begin
  if btnZoomIn.Down then
  begin
    btnZoomIn.Down := False;
    Shape1.Width := 0;
    Shape1.Height := 0;
    Paintbox1.Cursor := crDefault;
    EndX := X - ScrollBox1.HorzScrollBar.Position;
    EndY := Y - ScrollBox1.VertScrollBar.Position;

    if StartX < EndX then
    begin
      LeftX := StartX + ScrollBox1.HorzScrollBar.Position;
      RightX := EndX + ScrollBox1.HorzScrollBar.Position ;
    end
    else
    begin
      LeftX := EndX + ScrollBox1.HorzScrollBar.Position;
      RightX := StartX + ScrollBox1.HorzScrollBar.Position;
    end;
    if StartY < EndY then
    begin
      TopY := StartY + ScrollBox1.VertScrollBar.Position;
      BottomY := EndY + ScrollBox1.VertScrollBar.Position;
    end
    else
    begin
      TopY := EndY + ScrollBox1.VertScrollBar.Position;
      BottomY := StartY + ScrollBox1.VertScrollBar.Position;
    end;

    XMult := (Paintbox1.Width)/(RightX-LeftX);
    YMult := (Paintbox1.Height)/(BottomY-TopY);

    APoint := TPoint.Create;
    try
      begin

        APoint.XCoord := LeftX ;
        APoint.YCoord := TopY;

        if XMult < YMult then
        begin
          LocalMult := XMult;
        end
        else
        begin
          LocalMult := YMult;
        end;
        Width := Round(Paintbox1.Width*LocalMult);
        Height := Round(Paintbox1.Height*LocalMult);
        if (Width > High(Smallint)) or (Height > High(Smallint)) then
        begin
          btnZoomIn.Enabled := False;
          if Width > High(Smallint) then
          begin
            LocalMult := (High(Smallint) -1)/Paintbox1.Width;
            Height := Round(Paintbox1.Height*LocalMult);
          end;
          if Height > High(Smallint) then
          begin
            LocalMult := (High(Smallint) -1)/Paintbox1.Height;
          end;
        end ;

        Paintbox1.Width := Round((Paintbox1.Width)*LocalMult);
        Paintbox1.Height := Round((Paintbox1.Height)*LocalMult);
        Multiplier := Multiplier *LocalMult;
        Scrollbox1.HorzScrollBar.Range := Paintbox1.Width;
        Scrollbox1.VertScrollBar.Range := Paintbox1.Height;
        ScrollBox1.HorzScrollBar.Position := APoint.XCoord;
        ScrollBox1.VertScrollBar.Position := APoint.YCoord;
      end;
    finally
      begin
        APoint.Free;
      end;
    end;

    Paintbox1.Invalidate;
  end
  else
  begin
    CurrentPoint := Select(X, Y);
    Paintbox1.Invalidate;
    if CurrentPoint <> nil then
    begin
      frmNodePosition := TfrmNodePosition.Create(Application);
      try
        begin
          frmNodePosition.adeX.Text := FloatToStr(CurrentPoint.X);
          frmNodePosition.adeY.Text := FloatToStr(CurrentPoint.Y);
          frmNodePosition.ShowModal;
        end
      finally
        begin
          frmNodePosition.Free;
        end;
      end;
    end;
  end;
end;

function TfrmEditContours.WriteContours: string;
var
  ContourStringList : TStringList;
  Index : integer;
  AContour : TContour;
  InnerIndex : integer;
begin
  ContourStringList := TStringList.Create;
  ContourStringList.Text := '';
  For index := 0 to ContourList.Count -1 do
  begin
    AContour := ContourList.Items[index];
    AContour.PointsToStrings;
    AContour.PointStrings.Add('');
    for InnerIndex := 0 to AContour.Heading.Count -1 do
    begin
        ContourStringList.Add(AContour.Heading[InnerIndex]);
    end;
    for InnerIndex := 0 to AContour.PointStrings.Count -1 do
    begin
      ContourStringList.Add(AContour.PointStrings[InnerIndex]);
    end;

//    ContourStringList.Text := ContourStringList.Text + AContour.Heading.Text + AContour.PointStrings.Text;
  end;

  result := '';
  For index := 0 to ContourStringList.Count -1 do
  begin
    result := result + ContourStringList.strings[index] + Chr(10);
  end;

  result := result + Chr(10);

  ContourStringList.Free;

end;

procedure TfrmEditContours.btnZoomInClick(Sender: TObject);
begin
  ZoomBegun := False;
  if btnZoomIn.Down then
  begin
    Paintbox1.Cursor := crCross;
  end
  else
  begin
    Paintbox1.Cursor := crDefault;
  end;
end;

procedure TfrmEditContours.btnZoomOutClick(Sender: TObject);
begin
  Scrollbox1.HorzScrollBar.Position := 0;
  Scrollbox1.VertScrollBar.Position := 0;
  Scrollbox1.HorzScrollBar.Visible := False;
  Scrollbox1.VertScrollBar.Visible := False;
  Paintbox1.Width := Scrollbox1.ClientWidth;
  Paintbox1.Height:= Scrollbox1.ClientHeight;
  Scrollbox1.HorzScrollBar.Range := Scrollbox1.ClientWidth;
  Scrollbox1.VertScrollBar.Range := Scrollbox1.ClientHeight;
  Scrollbox1.HorzScrollBar.Visible := True;
  Scrollbox1.VertScrollBar.Visible := True;
  GetMinMax;
  Multiplier := DefaultMultiplier;
  btnZoomIn.Enabled := True;
  Paintbox1.Invalidate;
end;

procedure TfrmEditContours.FormResize(Sender: TObject);
begin
  Paintbox1.Invalidate;

end;

procedure TfrmEditContours.PaintBox1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if btnZoomIn.Down then
  begin
    ZoomBegun := True;
    StartX := X - ScrollBox1.HorzScrollBar.Position;
    StartY := Y - ScrollBox1.VertScrollBar.Position;
  end;
end;

procedure TfrmEditContours.PaintBox1MouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var
  APoint : TPoint;
begin
  APoint := TPoint.Create;
  try
    APoint.XCoord := X;
    APoint.YCoord := Y;
    Label1.Caption := 'X: ' +FloatToStrF(APoint.X, ffNumber, 7, 4);
    Label2.Caption := 'Y: ' +FloatToStrF(APoint.Y, ffNumber, 7, 4);
  finally
    APoint.Free;
  end;


  if btnZoomIn.Down and ZoomBegun then
  begin
    EndX := X - ScrollBox1.HorzScrollBar.Position;
    EndY := Y - ScrollBox1.VertScrollBar.Position;
    if StartX < EndX then
    begin
      Shape1.Left := StartX;
      Shape1.Width := EndX - StartX;
    end
    else
    begin
      Shape1.Left := EndX;
      Shape1.Width := StartX - EndX;
    end;
    if StartY < EndY then
    begin
      Shape1.Top := StartY;
      Shape1.Height := EndY - StartY;
    end
    else
    begin
      Shape1.Top := EndY;
      Shape1.Height := StartY - EndY;
    end;
  end;

end;

procedure TfrmEditContours.ReverseContours;
var
  ContourIndex,PointIndex : Integer;
  AContour : TContour;
begin
  for ContourIndex := 0 to Contourlist.Count -1 do
  begin
    AContour := Contourlist[ContourIndex];
    for PointIndex := 0 to Round((AContour.FPoints.Count-2)/2) do
    begin
      AContour.FPoints.Exchange(PointIndex, AContour.FPoints.Count-PointIndex-1);
    end;
  end;

end;

procedure GetANEFunctions( var numNames : ANE_INT32;
          var descriptors : ANEPIEDescPtr ); cdecl;
begin
        {$ASSERTIONS ON}
        {Assertions are a debugging tool. They should be turned off
        in the final version. They are useful for "Just-in-time" debugging
        with Turbo-Debugger 32. See Delphi help for more information.}

	numNames := 0;
	gEditContoursPIEImportPIEDesc.version := IMPORT_PIE_VERSION;
	gEditContoursPIEImportPIEDesc.name := 'Edit Contours';   // name of project
	gEditContoursPIEImportPIEDesc.importFlags := kImportFromLayer;
 	gEditContoursPIEImportPIEDesc.fromLayerTypes :=
          kPIEInformationLayer or kPIEDomainLayer ;
 	gEditContoursPIEImportPIEDesc.toLayerTypes :=
          kPIEInformationLayer or kPIEDomainLayer  ;
 	gEditContoursPIEImportPIEDesc.doImportProc := @GEditContoursPIE;// address of Post Processing Function function

	// prepare PIE descriptor for Example Delphi PIE
	gEditContoursPIEDesc.name := 'Edit Contours';      // PIE name
	gEditContoursPIEDesc.PieType := kImportPIE;                   // PIE type: project PIE
	gEditContoursPIEDesc.descriptor := @gEditContoursPIEImportPIEDesc;	// pointer to descriptor

	Assert (numNames < kMaxFunDesc) ;
        gFunDesc[numNames] := @gEditContoursPIEDesc;
        Inc(numNames);	// add descriptor to list

	gReverseContoursPIEImportPIEDesc.version := IMPORT_PIE_VERSION;
	gReverseContoursPIEImportPIEDesc.name := 'Revese Contours on Clipboard';   // name of project
	gReverseContoursPIEImportPIEDesc.importFlags := kImportFromLayer;
 	gReverseContoursPIEImportPIEDesc.fromLayerTypes :=
          kPIEInformationLayer or kPIEDomainLayer ;
 	gReverseContoursPIEImportPIEDesc.toLayerTypes :=
          kPIEInformationLayer or kPIEDomainLayer  ;
 	gReverseContoursPIEImportPIEDesc.doImportProc := @GReverseContoursPIE;// address of Post Processing Function function

	// prepare PIE descriptor for Example Delphi PIE
	gReverseContoursPIEDesc.name := 'Revese Contours on Clipboard';      // PIE name
	gReverseContoursPIEDesc.PieType := kImportPIE;                   // PIE type: project PIE
	gReverseContoursPIEDesc.descriptor := @gReverseContoursPIEImportPIEDesc;	// pointer to descriptor

	Assert (numNames < kMaxFunDesc) ;
        gFunDesc[numNames] := @gReverseContoursPIEDesc;
        Inc(numNames);	// add descriptor to list

        // Data to contours
	gData2ContoursPIEImportPIEDesc.version := IMPORT_PIE_VERSION;
	gData2ContoursPIEImportPIEDesc.name := 'Data to Contour';   // name of project
	gData2ContoursPIEImportPIEDesc.importFlags := kImportFromLayer;
 	gData2ContoursPIEImportPIEDesc.fromLayerTypes :=
          kPIEDataLayer ;
 	gData2ContoursPIEImportPIEDesc.toLayerTypes :=
          kPIEInformationLayer  ;
 	gData2ContoursPIEImportPIEDesc.doImportProc := @GData2ContourPIE;// address of Post Processing Function function

	// prepare PIE descriptor for Example Delphi PIE
	gData2ContoursPIEDesc.name := 'Data to Contour';      // PIE name
	gData2ContoursPIEDesc.PieType := kImportPIE;                   // PIE type: project PIE
	gData2ContoursPIEDesc.descriptor := @gData2ContoursPIEImportPIEDesc;	// pointer to descriptor

	Assert (numNames < kMaxFunDesc) ;
        gFunDesc[numNames] := @gData2ContoursPIEDesc;
        Inc(numNames);	// add descriptor to list



	descriptors := @gFunDesc;
end;

end.
