unit ExtractUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ArgusDataEntry, Menus, Grids, Buttons, TeeProcs, TeEngine,
  Chart, ExtCtrls, Series, DataGrid, ComCtrls;

type
  TfrmExtract = class(TForm)
    SaveDialog1: TSaveDialog;
    OpenDialog1: TOpenDialog;
    MainMenu1: TMainMenu;
    About1: TMenuItem;
    Panel1: TPanel;
    Chart1: TChart;
    Help1: TMenuItem;
    Panel2: TPanel;
    lblCount: TLabel;
    btnRead: TButton;
    btnSave: TButton;
    btnCancel: TButton;
    BitBtn1: TBitBtn;
    Splitter1: TSplitter;
    adeCellCount: TArgusDataEntry;
    File1: TMenuItem;
    ReadCellLocations1: TMenuItem;
    ReadHeadsorDrawdown1: TMenuItem;
    SaveCellLocations1: TMenuItem;
    SaveHeadsorDrawdown1: TMenuItem;
    SaveDialog2: TSaveDialog;
    dgDataPoints: TDataGrid;
    cbPlot: TCheckBox;
    OpenDialog2: TOpenDialog;
    rgProgramChoice: TRadioGroup;
    RichEdit1: TRichEdit;
    rgMOC3D: TRadioGroup;
    rgSutra: TRadioGroup;
    dgDataNodes: TDataGrid;
    procedure btnSaveClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnReadClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure sgDataPointsSetEditText(Sender: TObject; ACol, ARow: Integer;
      const Value: String);
    procedure sgDataPointsSelectCell(Sender: TObject; Col, Row: Integer;
      var CanSelect: Boolean);
    procedure adeCellCountChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Help1Click(Sender: TObject);
    procedure ReadCellLocations1Click(Sender: TObject);
    procedure SaveCellLocations1Click(Sender: TObject);
    procedure dgDataPointsSetEditText(Sender: TObject; ACol, ARow: Integer;
      const Value: String);
    procedure rgProgramChoiceClick(Sender: TObject);
    procedure rgMOC3DClick(Sender: TObject);
    procedure dgDataNodesSetEditText(Sender: TObject; ACol, ARow: Integer;
      const Value: String);
    procedure rgSutraClick(Sender: TObject);
  private
    function GetStringBetween(var AString : String; Before, After: String): string;
    function GetNextLine(SearchTerm: string; Start: integer;
      AStringList: TStringList): integer;
    procedure ReadFile;
    { Private declarations }
  public
    CancelProcess : boolean;
    CurrentText : string;
    SeriesList : TList;
    procedure ClearSeriesList;
    procedure ReadCellLocations(FileName: string);
    procedure ReadModflowFile;
    procedure ReadMt3dObservationsFile;
    procedure ReadMoc3dObservationsFile;
    procedure ReadSutraFile;
    procedure ReadNewSutraFile;
    { Public declarations }
  end;

var
  frmExtract: TfrmExtract;

implementation

uses AboutUnit, HelpUnit;

{$R *.DFM}

var
  DecimalChar : Char;

function InternationalStrToFloat(Value : string) : extended;
var
  DecimalLocation : integer;
begin
  if DecimalChar = '.' then
  begin
    DecimalLocation := Pos(',',Value);
    if DecimalLocation > 0 then
    begin
      Value[DecimalLocation] := '.';
    end;
  end;
  if DecimalChar = ',' then
  begin
    DecimalLocation := Pos('.',Value);
    if DecimalLocation > 0 then
    begin
      Value[DecimalLocation] := ',';
    end;
  end;
  Result := StrToFloat(Value);
end;


Procedure TfrmExtract.ReadModflowFile;
const
  HeadLabel = 'HEAD';
  DrawdownLabel = 'DRAWDOWN';
var
  ModflowOutput : TextFile; // the formatted head or drawdown file
  DescriptorLine : string;  // line describing the model characteristics.
  ADatum : double;  // a number read from the formatted head or drawdown file
  AString : string;
  TOTIM : double; // elapsed time
  NCOL,NROW,ILAY: integer; // the number of columns, number of rows and layer number
  EndText : integer;
  ColIndex, RowIndex : integer; // indexes to the colums and rows
  PointsIndex : integer; // an index to the point for which a hydrograph is to be extracted.
  DataString : String;  // This is used to store the formatted data that will be placed on a line of the TMemo
  FirstTime : boolean; // we don't need to put data in the memo at the beginning of the first time step.
  ATime : double; // elapsed time for the current time step
  CaptionString : string;
  ASeries : TLineSeries; // a hydrograph plotted on the chart
  RealArray : array of double; // an array of values read from
  RealArrayLength : integer;
  ChartIndex : integer;
  AChartSeries : TChartSeries;
//  CharCount, Index : integer;
//  AParagraph : TParaAttributes;
  Lines : TStringlist;
begin
//  Put the name of the file on the caption of the form.
  Caption := 'Hydrograph Extractor: ' + OpenDialog1.FileName;
  // disable thing that shouldn't be changed while reading the file.
  btnSave.Enabled := True;
  btnCancel.Enabled := True;
  btnRead.Enabled := False;
  adeCellCount.Enabled := False;
  dgDataPoints.Enabled := False;
  Lines := TStringlist.Create;
  try
  begin
    // ClearSeriesList destroys all the TLineSeries on the chart and
    // clears SeriesList
    ClearSeriesList;

    // initialize CancelProcess. CancelProcess will be tested while the
    // file is being read to see if the user has pressed the cancel button.
    CancelProcess := False;

    // initialize the memo
//    Memo1.Clear;
    RichEdit1.Clear;
//    Memo1.Lines.Add(Chr(9) + '(Column,Row,Layer)');
    Lines.Add(Chr(9) + '(Column,Row,Layer)');

    // create a headers for the data in the memo and create a TLineSeries
    // for each MODFLOW cell.
    AString := 'Time';
    For RowIndex := 1 to dgDataPoints.RowCount -1 do
    begin
      CaptionString := '(' + dgDataPoints.Cells[0,RowIndex] + ',' +
                                          dgDataPoints.Cells[1,RowIndex] + ',' +
                                          dgDataPoints.Cells[2,RowIndex] + ')';
      AString := AString + Chr(9) + CaptionString;

      ASeries := TLineSeries.Create(Chart1);
      ASeries.ParentChart := Chart1;
      SeriesList.Add(ASeries);
      ASeries.Title := CaptionString;
      ASeries.Pointer.Visible := True;
      ASeries.Active := cbPlot.Checked and (dgDataPoints.Cells[3, RowIndex]
        = dgDataPoints.Columns[3].PickList.Strings[1]);
    end;
//    Memo1.Lines.Add(AString);
    Lines.Add(AString);

    // this may take a while so change the cursor.
    Screen.Cursor := crHourGlass;

    // This opens the formatted head or drawdown file.
    AssignFile(ModflowOutput, OpenDialog1.FileName);

    // create a list of reals used for temporary storage of data read from the
    // file.
//    ARealList := TRealList.Create;

//    RealArray
    RealArrayLength := StrToInt(frmExtract.adeCellCount.Text);
    SetLength(RealArray, RealArrayLength);
    // initialize ATime
    ATime := 0;
    try
      begin
        // start reading the MODFLOW file at the beginning
        Reset(ModflowOutput) ;

        // we don't need to do something on the first time step that
        // we need to do on the others to initialize a variable that
        // can be checked to see if it is the first time step.
        FirstTime := True;

        // initialize the time.
        ATime := 0;
        while not Eof(ModflowOutput) do
        begin
          // if the user has pressed the cancel button, stop reading the file.
          Application.ProcessMessages;
          if CancelProcess then
          begin
            break;
          end;
          // read the descriptor line at the beginning of the array
          Readln(ModflowOutput, DescriptorLine);
          // if the line is empty you've reached the end of the file so stop.
          if DescriptorLine = '' then
          begin
            break;
          end;
          // check if this is a head or drawdown file.
          // quit if it is neither.
          IF (Pos(HeadLabel,DescriptorLine) > 0)
          then
            begin
              EndText := Pos(HeadLabel,DescriptorLine)+Length(HeadLabel);
//              TEXT := Copy(DescriptorLine,42,EndText -43);
            end
          else if Pos(DrawdownLabel,DescriptorLine) > 0
          then
            begin
              EndText := Pos(DrawdownLabel,DescriptorLine)+Length(DrawdownLabel);
//              TEXT := Copy(DescriptorLine,42,EndText -43);
            end
          else
            begin
              Beep;
              ShowMessage('Unable to read file');
              break;
            end;
          // get the elapsed time
          AString := Copy(DescriptorLine,27,15);
          TOTIM := InternationalStrToFloat(AString);
          // get the number of columns
          AString := Copy(DescriptorLine,EndText,6);
          NCOL := StrToInt(AString);
          // get the number of rows.
          AString := Copy(DescriptorLine,EndText+6,6);
          NROW := StrToInt(AString);
          // get the layer number
          AString := Copy(DescriptorLine,EndText+12,6);
          ILAY := StrToInt(AString);

          // If data for a time step has already been read,
          // add that data to RichEdit1.
          if (ILAY = 1) and not FirstTime then
          begin
            DataString := FloatToStr(ATime);
            for PointsIndex := 0 to RealArrayLength {ARealList.Count} -1 do
            begin
              DataString := DataString + Chr(9) + FloatToStr(RealArray[PointsIndex]);
            end;
            Lines.Add(DataString);
          end;

          // on the next time step we'll need to do the above procedure
          FirstTime := False;

          // Set the time to the total elapased time.
          ATime := TOTIM;
          For RowIndex := 1 to NROW do
          begin
            // if the user pressed the cancel button, quit.
            Application.ProcessMessages;
            if CancelProcess then
            begin
              break;
            end;
            For ColIndex := 1 to NCOL do
            begin
              // if the user pressed the cancel button, quit.
              Application.ProcessMessages;
              if CancelProcess then
              begin
                break;
              end;
              // don't try to read past the end of the file.
              if not Eof(ModflowOutput)
              then
                begin
                  // allow the application to respond to the user pressing the cancel button.
                  Application.ProcessMessages;
                  // if the user pressed the cancel button, quit.
                  if CancelProcess then
                  begin
                    break;
                  end;
                  // read a number from the MODFLOW file
                  Read(ModflowOutput, ADatum);
                  // Check if this is one of the data point we need to check.
                  // (This could be made faster by moving the String to integer
                  // conversion outside the loops.)
                  for PointsIndex := 1 to dgDataPoints.RowCount -1 do
                  begin
                    if (ColIndex = StrToInt(dgDataPoints.Cells[0,PointsIndex])) and
                       (RowIndex = StrToInt(dgDataPoints.Cells[1,PointsIndex])) and
                       (ILAY = StrToInt(dgDataPoints.Cells[2,PointsIndex])) then
                    begin
                      // store the data value temporarily in ARealList
//                      AReal := ARealList.Items[PointsIndex -1];
//                      AReal.Value := ADatum;

                      RealArray[PointsIndex -1] := ADatum;
                      // Add the data point to the appropriate TLineSeries
                      ASeries := SeriesList[PointsIndex -1];
                      ASeries.AddXY(ATime, ADatum, '', clTeeColor);
                    end;
                  end;
                end
              else
                begin
                  // quit if you've reached the end of the file
                  break;
                end;
            end;
          end;
          // read the rest of the last line after the last number in the array.
          Readln(ModflowOutput, AString);
        end;
      end;


    finally
      begin
        // store the data for the last time step in the memo.
        DataString := FloatToStr(ATime);
        for PointsIndex := 0 to RealArrayLength {ARealList.Count} -1 do
        begin
          DataString := DataString + Chr(9) + FloatToStr(RealArray[PointsIndex]);
        end;
        Lines.Add(DataString);
        // close the MODFLOW file
        CloseFile(ModflowOutput);

        for ChartIndex := 0 to Chart1.SeriesList.Count -1 do
        begin
          AChartSeries := Chart1.Series[ChartIndex];
          AChartSeries.Active := dgDataPoints.Cells[3, ChartIndex+1]
            = dgDataPoints.Columns[3].PickList.Strings[1];
        end;


        // restore the cursor
        Screen.Cursor := crDefault;
        RichEdit1.Lines := Lines;

      end;
    end;
  end;
  finally
    // enable or disable controls as appropriate.
    btnCancel.Enabled := False;
    btnRead.Enabled := True;
    adeCellCount.Enabled := True;
    dgDataPoints.Enabled := True;
    Lines.Free;
  end;

end;

Procedure TfrmExtract.ReadMt3dObservationsFile;
const
  HeadLabel = 'HEAD';
  DrawdownLabel = 'DRAWDOWN';
var
  Mt3dObservations : TextFile; // the formatted head or drawdown file
  ADatum : double;  // a number read from the formatted head or drawdown file
  AString : string;
  RowIndex : integer; // indexes to the colums and rows
  PointsIndex : integer; // an index to the point for which a hydrograph is to be extracted.
  DataString : String;  // This is used to store the formatted data that will be placed on a line of the TMemo
  ATime : double; // elapsed time for the current time step
  CaptionString : string;
  ASeries : TLineSeries; // a hydrograph plotted on the chart
  RealArray : array of double; // an array of values read from
  RealArrayLength : integer;
  ALine : string;
  FoundEnd : boolean;
  LineIndex : integer;
  count : integer;
  TimeStep : integer;
  Lines : TStringList;
begin
  dgDataPoints.Visible := True;
//  Put the name of the file on the caption of the form.
  Caption := 'Hydrograph Extractor: ' + OpenDialog1.FileName;
  // disable thing that shouldn't be changed while reading the file.
  btnSave.Enabled := True;
  btnCancel.Enabled := True;
  btnRead.Enabled := False;
  adeCellCount.Enabled := False;
  dgDataPoints.Enabled := False;
  Lines := TStringList.Create;
  try
  begin
    // ClearSeriesList destroys all the TLineSeries on the chart and
    // clears SeriesList
    ClearSeriesList;

    // initialize CancelProcess. CancelProcess will be tested while the
    // file is being read to see if the user has pressed the cancel button.
    CancelProcess := False;

    // initialize the memo
    RichEdit1.Clear;
    Lines.Add(Chr(9) + '(Column,Row,Layer)');

    // This opens the formatted head or drawdown file.
    AssignFile(Mt3dObservations, OpenDialog1.FileName);
    try
          // start reading the MODFLOW file at the beginning
          Reset(Mt3dObservations) ;

          // read title line
          readln(Mt3dObservations, ALine);
          Count := 0;

          LineIndex := 0;
          repeat
            begin
              readln(Mt3dObservations, ALine);
              Inc(Count);
              FoundEnd := (ALine[6] <> ' ');
              if not FoundEnd then
              begin
                ALine := Copy(ALine,20,Length(ALine));
                While Length(ALine) > 0 do
                begin
                  Inc(LineIndex);
                  adeCellCount.Text := IntToStr(LineIndex);
                  adeCellCountChange(adeCellCount);
                  dgDataPoints.Cells[2,LineIndex] := Trim(Copy(ALine,1,4));
                  dgDataPoints.Cells[1,LineIndex] := Trim(Copy(ALine,5,4));
                  dgDataPoints.Cells[0,LineIndex] := Trim(Copy(ALine,9,4));
                  ALine := Copy(ALine,15,Length(ALine));
                  dgDataPoints.Cells[3,LineIndex] := dgDataPoints.Columns[3].PickList[0];
                end;
              end;
            end
          until FoundEnd;


    finally
        CloseFile(Mt3dObservations);
    end;
    try
      // start reading the MODFLOW file at the beginning
      Reset(Mt3dObservations) ;

      for LineIndex := 1 to Count do
      begin
        readln(Mt3dObservations, ALine);
      end;

      // create a headers for the data in the memo and create a TLineSeries
      // for each MODFLOW cell.
      AString := 'Time';
      For RowIndex := 1 to dgDataPoints.RowCount -1 do
      begin
        CaptionString := '(' + dgDataPoints.Cells[0,RowIndex] + ',' +
                                            dgDataPoints.Cells[1,RowIndex] + ',' +
                                            dgDataPoints.Cells[2,RowIndex] + ')';
        AString := AString + Chr(9) + CaptionString;

        ASeries := TLineSeries.Create(Chart1);
        ASeries.ParentChart := Chart1;
        SeriesList.Add(ASeries);
        ASeries.Title := CaptionString;
        ASeries.Pointer.Visible := True;
        ASeries.Active := cbPlot.Checked and (dgDataPoints.Cells[3, RowIndex]
          = dgDataPoints.Columns[3].PickList.Strings[1]);
      end;
//      Memo1.Lines.Add(AString);
      Lines.Add(AString);

      // this may take a while so change the cursor.
      Screen.Cursor := crHourGlass;


      RealArrayLength := StrToInt(frmExtract.adeCellCount.Text);
      SetLength(RealArray, RealArrayLength);
      try
        begin

          // we don't need to do something on the first time step that
          // we need to do on the others to initialize a variable that
          // can be checked to see if it is the first time step.
          while not Eof(Mt3dObservations) do
          begin
            // if the user has pressed the cancel button, stop reading the file.
            Application.ProcessMessages;
            if CancelProcess then
            begin
              break;
            end;


            Read(Mt3dObservations, TimeStep);
            Read(Mt3dObservations, ATime);

            for RowIndex := 1 to dgDataPoints.RowCount -1 do
            begin
              Read(Mt3dObservations, ADatum);
              RealArray[RowIndex -1] := ADatum;
              // Add the data point to the appropriate TLineSeries
              ASeries := SeriesList[RowIndex -1];
              ASeries.AddXY(ATime, ADatum, '', clTeeColor);
            end;
            // read the rest of the last line after the last number in the array.
            Readln(Mt3dObservations, AString);

            // add that data to RichEdit1.
            DataString := FloatToStr(ATime);
            for PointsIndex := 0 to RealArrayLength  -1 do
            begin
              DataString := DataString + Chr(9) + FloatToStr(RealArray[PointsIndex]);
            end;
//            Memo1.Lines.Add(DataString);
            Lines.Add(DataString);

          end;
        end;


      finally
        begin
          RichEdit1.Lines := Lines;
          // restore the cursor
          Screen.Cursor := crDefault;
        end;
      end;
    finally
        CloseFile(Mt3dObservations);
    end;
  end;
  finally
    // enable or disable controls as appropriate.
    btnCancel.Enabled := False;
    btnRead.Enabled := True;
    adeCellCount.Enabled := True;
    dgDataPoints.Enabled := True;
    Lines.Free;
  end;

end;

procedure TfrmExtract.ReadFile;
begin
    case rgProgramChoice.ItemIndex of
    0:
      begin
        ReadModflowFile;
        // This is placed in a separate procedure to facilitate
        // updating the program to read data from programs other than the
        // currently supported programs.
      end;
    1:
      begin
        ReadMt3dObservationsFile;
        // This is placed in a separate procedure to facilitate
        // updating the program to read data from programs other than the
        // currently supported programs.
      end;
    2:
      begin
        ReadMoc3dObservationsFile;
        // This is placed in a separate procedure to facilitate
        // updating the program to read data from programs other than the
        // currently supported programs.
      end;
    3:
      begin
        ReadSutraFile;
//        ReadNewSutraFile;
        // This is placed in a separate procedure to facilitate
        // updating the program to read data from programs other than the
        // currently supported programs.
      end;
    else
      begin
        Assert(False);
      end;
    end;
end;


procedure TfrmExtract.btnReadClick(Sender: TObject);
begin
  if OpenDialog1.Execute then
  begin
    ReadFile;
  end;
end;

procedure TfrmExtract.btnSaveClick(Sender: TObject);
begin
  // save the memo to a file.
  if SaveDialog1.Execute then
  begin
    Screen.Cursor := crHourGlass;
    try
      RichEdit1.Lines.SaveToFile(SaveDialog1.FileName);
    finally
      Screen.Cursor := crDefault;
    end;
  end;
end;

procedure TfrmExtract.About1Click(Sender: TObject);
begin
  // show the "About" form
  frmAbout.ShowModal;
end;

procedure TfrmExtract.btnCancelClick(Sender: TObject);
begin
  // CancelProcess is checked while reading a file. The program will
  // stop reading the file if CancelProcess is true.
  CancelProcess := True;
end;

procedure TfrmExtract.ReadCellLocations(FileName: string);
var
  RowIndex : integer;
  ColIndex : integer;
  DataFile : TextFile;
  RowCount : integer;
  AnIndex : integer;
  AString : string;
begin

  if FileExists(FileName) then
  begin
    if rgProgramChoice.ItemIndex = 0 then
    begin
      AssignFile(DataFile, FileName);
      Reset(DataFile);
      try
        // read the number of cells.
        if Not Eof(DataFile) then
        begin
          read(DataFile,RowCount);
          adeCellCount.Text := IntToStr(RowCount);
        end;

        // read the row, column, and layer numbers.
          for RowIndex := 1 to dgDataPoints.RowCount -1 do
          begin
            for ColIndex := 0 to dgDataPoints.ColCount -2 do
            begin
              if Not Eof(DataFile) then
              begin
                read(DataFile,AnIndex);
                if Not Eof(DataFile) then
                begin
                  dgDataPoints.Cells[ColIndex, RowIndex] := IntToStr(AnIndex);
                end;
              end;
            end;
            if Not Eof(DataFile) then
            begin
              ReadLn(DataFile,AString);
              AString := Trim(AString);
              if AString <> '' then
              dgDataPoints.Cells[3, RowIndex] := AString;
            end;

          end;
      finally
        // close the file.
        CloseFile(DataFile)
      end;
    end;
  end;
end;

procedure TfrmExtract.FormCreate(Sender: TObject);
var
  FileName : string;
begin
  // prevent the top panel from getting to small.
  Panel1.Constraints.MinHeight := Panel1.Height;
  // create the list of TLineSeries
  SeriesList := TList.Create;
  // initialize the text in the string grid.

  dgDataPoints.Cells[0, 1] := '1';
  dgDataPoints.Cells[1, 1] := '1';
  dgDataPoints.Cells[2, 1] := '1';
  dgDataPoints.Cells[3, 1] := dgDataPoints.Columns[3].PickList.Strings[1];
  // See if there are any command line parameters.
  if ParamCount > 0 then
  begin
    if ParamCount > 2 then
    begin
      rgProgramChoice.ItemIndex := StrToInt(ParamStr(3));
    end;
    // get the first command line parameter
    FileName := ParamStr(1);
    // if the filename in the first command line parameter represents a
    // file that exists, open that file and read the data into the program
    ReadCellLocations(FileName);
    if FileExists(FileName) or (rgProgramChoice.ItemIndex > 0) then
    begin
      // check if there are at least two command line parameters.
      if ParamCount > 1 then
      begin
        // Get the second command line parameter.
        FileName := ParamStr(2);
        // read the file if it exists.
        if FileExists(FileName) then
        begin
          OpenDialog1.FileName := FileName;
          ReadFile;
        end;
      end;

    end;
  end;
end;

procedure TfrmExtract.sgDataPointsSetEditText(Sender: TObject; ACol,
  ARow: Integer; const Value: String);
var
  AValue : integer;
begin
  // test that the user has entered a valid number. If not, restore the cell to
  // what was in it before. (See sgDataPointsSelectCell.)
  Try
    begin
      if (ARow > 0) and not (Value = '') then
      begin
        AValue := StrToInt(dgDataPoints.Cells[ACol, ARow]);
        If AValue < 1 then
        begin
          Beep;
          dgDataPoints.Cells[ACol, ARow] := CurrentText;
        end;
      end;
    end
  except on EConvertError do
    begin
        Beep;
        dgDataPoints.Cells[ACol, ARow] := CurrentText;
    end;
  end;
end;

procedure TfrmExtract.sgDataPointsSelectCell(Sender: TObject; Col,
  Row: Integer; var CanSelect: Boolean);
begin
  // temporily store the text in the current cell
  CurrentText := dgDataPoints.Cells[Col, Row];
end;

procedure TfrmExtract.adeCellCountChange(Sender: TObject);
var
  OldRowCount : integer;
  Index : integer;
begin
  // change the number of rows in response to a change in the number of cells
  // that a user has specified.
  if not (csLoading in frmExtract.ComponentState) then
  begin
    OldRowCount := dgDataPoints.RowCount;
    dgDataPoints.RowCount := StrToInt(adeCellCount.Text) + 1;
    dgDataNodes.RowCount := dgDataPoints.RowCount;
    if dgDataPoints.RowCount > OldRowCount then
    begin
      for Index := OldRowCount to dgDataPoints.RowCount -1 do
      begin
        dgDataPoints.Cells[0,Index] := '1';
        dgDataPoints.Cells[1,Index] := '1';
        dgDataPoints.Cells[2,Index] := '1';
        dgDataPoints.Cells[3, Index]
          := dgDataPoints.Columns[3].PickList.Strings[1];
          
        dgDataNodes.Cells[0,Index] := '1';
        dgDataNodes.Cells[1, Index]
          := dgDataNodes.Columns[1].PickList.Strings[1];
      end;
    end;
  end;

end;

procedure TfrmExtract.ClearSeriesList;
var
  Index : integer;
begin
  // destroy the series in SeriesList and clear SeriesList.
  for Index := 0 to SeriesList.Count -1 do
  begin
    TLineSeries(SeriesList[Index]).Free;
  end;
  SeriesList.Clear;
end;

procedure TfrmExtract.FormDestroy(Sender: TObject);
begin
  // Destroy SeriesList.
  SeriesList.Free;
end;

procedure TfrmExtract.Help1Click(Sender: TObject);
begin
  // Show frmHelp
  frmHelp.ShowModal;
end;

procedure GetDecimalChar;
var
  CommaSeparator : array[0..255] of Char;
begin
  GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SDECIMAL,@CommaSeparator,255);
  DecimalChar := CommaSeparator[0];
end;

procedure TfrmExtract.ReadCellLocations1Click(Sender: TObject);
begin
  if OpenDialog2.Execute then
  begin
    SaveDialog2.FileName := OpenDialog2.FileName;
    ReadCellLocations(OpenDialog2.FileName);
  end;
end;

procedure TfrmExtract.SaveCellLocations1Click(Sender: TObject);
Var
  AStringList : TStringList;
  Index : integer;
begin
  if SaveDialog2.Execute then
  begin
    AStringList := TStringList.Create;
    try
      AStringList.Add(adeCellCount.Text);
      for Index := 1 to dgDataPoints.RowCount -1 do
      begin
        AStringList.Add(dgDataPoints.Cells[0,Index] + ' ' +
          dgDataPoints.Cells[1,Index] + ' ' +
          dgDataPoints.Cells[2,Index] + ' ' +
          dgDataPoints.Cells[3,Index]);
      end;
      AStringList.SaveToFile(SaveDialog2.FileName);
    finally
      AStringList.Free;
    end;

  end;

end;

procedure TfrmExtract.dgDataPointsSetEditText(Sender: TObject; ACol,
  ARow: Integer; const Value: String);
var
  ASeries : TChartSeries;
  ChartIndex : integer;
begin
  if (ACol = 3) and (ARow > 0) then
  begin
    if rgProgramChoice.ItemIndex = 2 then
    begin
      ChartIndex := (ARow -1)*2 + rgMOC3D.ItemIndex;
      if Chart1.SeriesList.Count > ChartIndex then
      begin
        ASeries := Chart1.Series[ChartIndex];
        ASeries.Active := dgDataPoints.Cells[3, ARow]
          = dgDataPoints.Columns[3].PickList.Strings[1];
      end;
    end
    else
    begin
      if Chart1.SeriesList.Count > ARow -1 then
      begin
        ASeries := Chart1.Series[ARow -1];
        ASeries.Active := dgDataPoints.Cells[3, ARow]
          = dgDataPoints.Columns[3].PickList.Strings[1];
      end;
    end;
  end;


end;

procedure TfrmExtract.rgProgramChoiceClick(Sender: TObject);
begin
  case rgProgramChoice.ItemIndex of
  0:
    begin
      OpenDialog1.Filter := 'formatted head files (*.fhd)|*.fhd|formatted '
        + 'drawdown files (*.fdn)|*.fdn|All Files|*.*';
      Chart1.LeftAxis.Title.Caption := 'head or drawdown';
      dgDataPoints.Visible := True;
      dgDataNodes.Visible := False;
      rgMOC3D.Enabled := False;
      rgSutra.Enabled := False;
      lblCount.Caption := 'Number of Cells';
    end;
  1:
    begin
      OpenDialog1.Filter := 'MT3D Observation files (*.obs)|*.obs|All Files|*.*';
      Chart1.LeftAxis.Title.Caption := 'concentration';
      dgDataPoints.Visible := False;
      dgDataNodes.Visible := False;
      rgMOC3D.Enabled := False;
      rgSutra.Enabled := False;
      lblCount.Caption := 'Number of Cells';
    end;
  2:
    begin
      OpenDialog1.Filter := 'MOC3D Observation files (*.oba)|*.oba|All Files|*.*';
      Chart1.LeftAxis.Title.Caption := LowerCase(rgMOC3D.Items[rgMOC3D.ItemIndex]);
      dgDataPoints.Visible := False;
      dgDataNodes.Visible := False;
      rgMOC3D.Enabled := True;
      rgSutra.Enabled := False;
      lblCount.Caption := 'Number of Cells';
    end;
  3:
    begin
//      OpenDialog1.Filter := 'SUTRA Listing files (*.d6)|*.d6|All Files|*.*';
      OpenDialog1.Filter := 'SUTRA Listing files (*.obs)|*.obs|All Files|*.*';
      Chart1.LeftAxis.Title.Caption := LowerCase(rgSutra.Items[rgSutra.ItemIndex]);
      dgDataPoints.Visible := False;
      dgDataNodes.Visible := False;
      rgMOC3D.Enabled := False;
      rgSutra.Enabled := True;
      lblCount.Caption := 'Number of Nodes';
    end;
  else
    begin
      Assert(False);
    end;
  end;

end;

procedure TfrmExtract.ReadMoc3dObservationsFile;
const
  HeadLabel = 'HEAD';
  DrawdownLabel = 'DRAWDOWN';
var
  Moc3dOutput : TextFile; // the formatted head or drawdown file
  ADatum : double;  // a number read from the formatted head or drawdown file
  AString : string;
  TOTIM : double; // elapsed time
  RowIndex : integer; // indexes to the colums and rows
  PointsIndex : integer; // an index to the point for which a hydrograph is to be extracted.
  DataString : String;  // This is used to store the formatted data that will be placed on a line of the TMemo
  ATime : double; // elapsed time for the current time step
  CaptionString : string;
  ASeries : TLineSeries; // a hydrograph plotted on the chart
  RealArray : array of double; // an array of values read from
  RealArrayLength : integer;
  ChartIndex : integer;
  AChartSeries : TChartSeries;
  Lines : TStringlist;
  ALine : string;
  LineIndex : integer;
  FoundEnd : boolean;
  Position : integer;
  SearchString : string;
  SearchStringLength : integer;
  CellIndex : integer;
  CorrectType : boolean;
  SeriesIndex : integer;
begin
//  Put the name of the file on the caption of the form.
  Caption := 'Hydrograph Extractor: ' + OpenDialog1.FileName;
  // disable thing that shouldn't be changed while reading the file.
  btnSave.Enabled := True;
  btnCancel.Enabled := True;
  btnRead.Enabled := False;
  adeCellCount.Enabled := False;
  dgDataPoints.Enabled := False;
  Lines := TStringlist.Create;
  try
  begin
    // ClearSeriesList destroys all the TLineSeries on the chart and
    // clears SeriesList
    ClearSeriesList;

    // initialize CancelProcess. CancelProcess will be tested while the
    // file is being read to see if the user has pressed the cancel button.
    CancelProcess := False;

    // initialize RichEdit1
    RichEdit1.Clear;
    Lines.Add(Chr(9) + 'Head and Concentration at (Column,Row,Layer)');


    AssignFile(Moc3dOutput, OpenDialog1.FileName);
    try
      // start reading the MOC3D file at the beginning
      Reset(Moc3dOutput) ;

      // read title lines
      readln(Moc3dOutput, ALine);
      readln(Moc3dOutput, ALine);
      readln(Moc3dOutput, ALine);

      LineIndex := 0;
      repeat
        begin
          SearchString := 'H & C AT';
          SearchStringLength := Length(SearchString);
          Position := Pos(SearchString,ALine);
          FoundEnd := Position=0;
          if not FoundEnd then
          begin
            ALine := Copy(ALine,Position+SearchStringLength,Length(ALine));
            if Length(ALine) > 0 then
            begin
              Inc(LineIndex);
              adeCellCount.Text := IntToStr(LineIndex);
              adeCellCountChange(adeCellCount);
              SearchString := ',';

              Position := Pos(SearchString,ALine);
              dgDataPoints.Cells[2,LineIndex] := Trim(Copy(ALine,1,Position-1));
              ALine := Trim(Copy(ALine,Position+1,Length(ALine)));

              Position := Pos(SearchString,ALine);
              dgDataPoints.Cells[1,LineIndex] := Trim(Copy(ALine,1,Position-1));
              ALine := Trim(Copy(ALine,Position+1,Length(ALine)));

              SearchString := ' ';
              Position := Pos(SearchString,ALine);
              dgDataPoints.Cells[0,LineIndex] := Trim(Copy(ALine,1,Position-1));
              ALine := Trim(Copy(ALine,Position+1,Length(ALine)));

              dgDataPoints.Cells[3,LineIndex] := dgDataPoints.Columns[3].PickList[0];
            end;
          end;
        end
      until FoundEnd;


    finally
        CloseFile(Moc3dOutput);
    end;




    // create a headers for the data in the memo and create a TLineSeries
    // for each MODFLOW cell.
    AString := 'Time';
    For RowIndex := 1 to dgDataPoints.RowCount -1 do
    begin
      CaptionString := '(' +
        dgDataPoints.Cells[0,RowIndex] + ',' +
        dgDataPoints.Cells[1,RowIndex] + ',' +
        dgDataPoints.Cells[2,RowIndex] + ')';
      AString := AString + Chr(9) + 'H & C AT' + Chr(9) + CaptionString;

      ASeries := TLineSeries.Create(Chart1);
      ASeries.ParentChart := Chart1;
      SeriesList.Add(ASeries);
      ASeries.Title := CaptionString;
      ASeries.Pointer.Visible := True;
      ASeries.Active := cbPlot.Checked and (dgDataPoints.Cells[3, RowIndex]
        = dgDataPoints.Columns[3].PickList.Strings[1])
        and (rgMOC3D.ItemIndex = 0);

      ASeries := TLineSeries.Create(Chart1);
      ASeries.ParentChart := Chart1;
      SeriesList.Add(ASeries);
      ASeries.Title := CaptionString;
      ASeries.Pointer.Visible := True;
      ASeries.Active := cbPlot.Checked and (dgDataPoints.Cells[3, RowIndex]
        = dgDataPoints.Columns[3].PickList.Strings[1])
        and (rgMOC3D.ItemIndex = 1);
    end;
    Lines.Add(AString);

    // this may take a while so change the cursor.
    Screen.Cursor := crHourGlass;

    // This opens the formatted head or drawdown file.
    AssignFile(Moc3dOutput, OpenDialog1.FileName);

    // create a list of reals used for temporary storage of data read from the
    // file.

//    RealArray
    RealArrayLength := StrToInt(frmExtract.adeCellCount.Text)*2;
    SetLength(RealArray, RealArrayLength);
    // initialize ATime
    try
      begin
        // start reading the MODFLOW file at the beginning
        Reset(Moc3dOutput) ;

        // read title lines
        readln(Moc3dOutput, ALine);
        readln(Moc3dOutput, ALine);
        readln(Moc3dOutput, ALine);


        while not Eof(Moc3dOutput) do
        begin
          // if the user has pressed the cancel button, stop reading the file.
          Application.ProcessMessages;
          if CancelProcess then
          begin
            break;
          end;
          // get the elapsed time
          read(Moc3dOutput, TOTIM);
          ATime := TOTIM;


          // Set the time to the total elapased time.
          For RowIndex := 1 to dgDataPoints.RowCount -1 do
          begin
            // if the user pressed the cancel button, quit.
            Application.ProcessMessages;
            if CancelProcess then
            begin
              break;
            end;
              // don't try to read past the end of the file.
              if not Eof(Moc3dOutput)
              then
                begin
                  // allow the application to respond to the user pressing the cancel button.
                  Application.ProcessMessages;
                  // if the user pressed the cancel button, quit.
                  if CancelProcess then
                  begin
                    break;
                  end;
                  // read a number from the MODFLOW file
                  Read(Moc3dOutput, ADatum);
                  // Check if this is one of the data point we need to check.
                  // (This could be made faster by moving the String to integer
                  // conversion outside the loops.)

                  SeriesIndex := (RowIndex -1)*2;

                  RealArray[SeriesIndex] := ADatum;
                  // Add the data point to the appropriate TLineSeries
                  ASeries := SeriesList[SeriesIndex];
                  ASeries.AddXY(ATime, ADatum, '', clTeeColor);

                  // read a number from the MODFLOW file
                  Read(Moc3dOutput, ADatum);
                  // Check if this is one of the data point we need to check.
                  // (This could be made faster by moving the String to integer
                  // conversion outside the loops.)

                  Inc(SeriesIndex); 

                  RealArray[SeriesIndex] := ADatum;
                  // Add the data point to the appropriate TLineSeries
                  ASeries := SeriesList[SeriesIndex];
                  ASeries.AddXY(ATime, ADatum, '', clTeeColor);
                end
              else
                begin
                  // quit if you've reached the end of the file
                  break;
                end;
          end;
          // read the rest of the last line after the last number in the array.
          Readln(Moc3dOutput, AString);

          // If data for a time step has already been read,
          // add that data to RichEdit1.
          DataString := FloatToStr(ATime);
          for PointsIndex := 0 to RealArrayLength -1 do
          begin
            DataString := DataString + Chr(9) + FloatToStr(RealArray[PointsIndex]);
          end;
          Lines.Add(DataString);


        end;
      end;


    finally
      begin
        // close the MODFLOW file
        CloseFile(Moc3dOutput);

        for ChartIndex := 0 to Chart1.SeriesList.Count -1 do
        begin
          AChartSeries := Chart1.Series[ChartIndex];
          CellIndex := (ChartIndex div 2) + 1;
          CorrectType := (Odd(ChartIndex) = Odd(rgMOC3D.ItemIndex));
          AChartSeries.Active := CorrectType
            and (dgDataPoints.Cells[3, CellIndex]
            = dgDataPoints.Columns[3].PickList.Strings[1]);
        end;

        // restore the cursor
        Screen.Cursor := crDefault;
        RichEdit1.Lines := Lines;

      end;
    end;
  end;
  finally
    // enable or disable controls as appropriate.
    btnCancel.Enabled := False;
    btnRead.Enabled := True;
    adeCellCount.Enabled := True;
    dgDataPoints.Visible := True;
    dgDataPoints.Enabled := True;
    Lines.Free;
  end;
end;

procedure TfrmExtract.rgMOC3DClick(Sender: TObject);
var
  ASeries : TChartSeries;
  ChartIndex, CellIndex : integer;
  CorrectType : boolean;
begin
  if rgProgramChoice.ItemIndex = 2 then
  begin
    Chart1.LeftAxis.Title.Caption := LowerCase(rgMOC3D.Items[rgMOC3D.ItemIndex]);
    for ChartIndex := 0 to Chart1.SeriesList.Count -1 do
    begin
      ASeries := Chart1.Series[ChartIndex];

      CellIndex := (ChartIndex div 2) + 1;
      CorrectType := (Odd(ChartIndex) = Odd(rgMOC3D.ItemIndex));

      ASeries.Active := CorrectType
        and (dgDataPoints.Cells[3, CellIndex]
        = dgDataPoints.Columns[3].PickList.Strings[1]);
    end;
  end


end;

procedure TfrmExtract.ReadSutraFile;
var
  SutraFile : TStringList; // the Sutra output file
  ADatum : double;  // a number read from the formatted head or drawdown file
  AString : string;
  RowIndex : integer; // indexes to the colums and rows
  PointsIndex : integer; // an index to the point for which a hydrograph is to be extracted.
  DataString : String;  // This is used to store the formatted data that will be placed on a line of the TMemo
  ATime : double; // elapsed time for the current time step
  CaptionString : string;
  ASeries : TLineSeries; // a hydrograph plotted on the chart
  RealArray : array of double; // an array of values read from
  RealArrayLength : integer;
  ChartIndex : integer;
  AChartSeries : TChartSeries;
  Lines : TStringlist;
  ALine : string;
  LineIndex : integer;
  SearchString : string;
  SearchStringLength : integer;
  CellIndex : integer;
  CorrectType : boolean;
  SeriesIndex, InnerSeriesIndex : integer;
  StartLine, StopLine, CaptionLine : integer;
  Captions : TStringList;
  ItemIndex : integer;
  Node, TimeStep : string;
  NodeIndex, TypeIndex : integer;
  TitleLine : string;
  ObservationStart : integer;
  MoreNodes : boolean;
  ThisCount : integer;
  FirstTime : boolean;
  DataIndex : integer;
begin
//  Put the name of the file on the caption of the form.
  Caption := 'Hydrograph Extractor: ' + OpenDialog1.FileName;
  // disable thing that shouldn't be changed while reading the file.
  btnSave.Enabled := True;
  btnCancel.Enabled := True;
  btnRead.Enabled := False;
  adeCellCount.Enabled := False;
  dgDataPoints.Enabled := False;
  SutraFile := TStringlist.Create;
  Lines := TStringlist.Create;
  // this may take a while so change the cursor.
  Screen.Cursor := crHourGlass;

  try
  begin
    // ClearSeriesList destroys all the TLineSeries on the chart and
    // clears SeriesList
    ClearSeriesList;

    // initialize CancelProcess. CancelProcess will be tested while the
    // file is being read to see if the user has pressed the cancel button.
    CancelProcess := False;

    // initialize RichEdit1
    RichEdit1.Clear;

    RowIndex := 0;
    SutraFile.LoadFromFile(OpenDialog1.FileName);
    try
      ObservationStart := GetNextLine(
        'O  B  S  E  R  V  A  T  I  O  N     N  O  D  E     D  A  T  A',
        0, SutraFile);

      if ObservationStart = -1 then
      begin
        MessageDlg(OpenDialog1.FileName + ' contains no observation nodes.',
          mtWarning, [mbOK], 0);
      end
      else
      begin
        if not CancelProcess then
        begin
          Captions := TStringList.Create;
          try
            LineIndex := GetNextLine('NODE',ObservationStart, SutraFile);

            MoreNodes := LineIndex > -1;
            While MoreNodes do
            begin
              ALine := SutraFile[LineIndex];

              While (Length(ALine) > 0) do
              begin
                Inc(RowIndex);
                adeCellCount.Text := IntToStr(RowIndex);
                adeCellCountChange(adeCellCount);
                Node := GetStringBetween(ALine,'NODE','NODE');
                dgDataNodes.Cells[0,RowIndex] := Node;
                dgDataNodes.Cells[1,RowIndex] := dgDataNodes.Columns[1].PickList[0];
  //              TitleLine := TitleLine + Chr(9) + 'Node ' + Node;
              end;

              CaptionLine := GetNextLine('TIME STEP',LineIndex, SutraFile);

              ALine := SutraFile[CaptionLine];
              GetStringBetween(ALine,'TIME(SEC)','');
              While Pos(' ',ALine) <> 0 do
              begin
                AString := GetStringBetween(ALine,'',' ');
                if Captions.IndexOf(AString) = -1 then
                begin
                  Captions.Add(AString);
                end;
              end;
              if (Captions.IndexOf(ALine) = -1) and (ALine <> '') then
              begin
                Captions.Add(ALine);
              end;
  //            ItemIndex := rgSutra.ItemIndex;
  //            rgSutra.Items := Captions;
  //            rgSutra.ItemIndex := ItemIndex;

              LineIndex := GetNextLine('NODE',LineIndex+1, SutraFile);
              MoreNodes := LineIndex > -1;
            end;
            ItemIndex := rgSutra.ItemIndex;
            rgSutra.Items := Captions;
            rgSutra.ItemIndex := ItemIndex;
          finally
            Captions.Free;
          end;

  {        StartLine := GetNextLine('TIME STEP',LineIndex, SutraFile)+1;
          StopLine := StartLine;
          for LineIndex := StartLine to SutraFile.Count -1 do
          begin
            StopLine := LineIndex;
            if Trim(SutraFile[LineIndex]) = '' then
            begin
              StopLine := LineIndex -1;
              break;
            end;
          end;
  }


          RealArrayLength := StrToInt(frmExtract.adeCellCount.Text)*rgSutra.Items.Count;
          SetLength(RealArray, RealArrayLength);

          TitleLine := ' TIME STEP' + Chr(9) + 'TIME(SEC)';
          For RowIndex := 1 to dgDataNodes.RowCount -1 do
          begin
            for TypeIndex := 0 to rgSutra.Items.Count -1 do
            begin
              CaptionString := dgDataNodes.Cells[0,RowIndex];
              TitleLine := TitleLine + Chr(9) + rgSutra.Items[TypeIndex]
                + ' Node ' + CaptionString;

              ASeries := TLineSeries.Create(Chart1);
              ASeries.ParentChart := Chart1;
              SeriesList.Add(ASeries);
              ASeries.Title := CaptionString;
              ASeries.Pointer.Visible := True;
              ASeries.Active := cbPlot.Checked and (dgDataNodes.Cells[1, RowIndex]
                = dgDataNodes.Columns[1].PickList.Strings[1])
                and (rgSutra.ItemIndex = TypeIndex);

            end;
          end;

          Lines.Add(TitleLine);




  //        RowIndex := 0;
          LineIndex := GetNextLine('NODE',ObservationStart, SutraFile);
          SeriesIndex := 0;

          FirstTime := True;
          MoreNodes := LineIndex > -1;
          While MoreNodes do
          begin
            ALine := SutraFile[LineIndex];
            ThisCount := 0;
            While (Length(ALine) > 0) do
            begin
              Inc(ThisCount);
              Node := GetStringBetween(ALine,'NODE','NODE');
            end;

            StartLine := GetNextLine('TIME STEP',LineIndex, SutraFile)+1;
            StopLine := StartLine;
            for LineIndex := StartLine to SutraFile.Count -1 do
            begin
              StopLine := LineIndex;
              ALine := Trim(SutraFile[LineIndex]);
              if (ALine = '') or (ALine = '1') then
              begin
                StopLine := LineIndex -1;
                break;
              end;
            end;


            DataIndex := 0;
            for LineIndex := StartLine to StopLine do
            begin
              Inc(DataIndex);
              Application.ProcessMessages;
              if CancelProcess then
              begin
                break;
              end;
              ALine := Trim(SutraFile[LineIndex]);
              TimeStep := GetStringBetween(ALine,'',' ');
              AString := GetStringBetween(ALine,'',' ');
              if Pos('D',AString) > 0 then
              begin
                AString[Pos('D',AString)] := 'E';
              end;
              if Pos('d',AString) > 0 then
              begin
                AString[Pos('d',AString)] := 'e';
              end;
              InnerSeriesIndex := -1;
              ATime := InternationalStrToFloat(AString);
              while Length(ALine) >0 do
              begin
                Inc(InnerSeriesIndex);
                AString := GetStringBetween(ALine,'',' ');
                if Pos('D',AString) > 0 then
                begin
                  AString[Pos('D',AString)] := 'E';
                end;
                if Pos('d',AString) > 0 then
                begin
                  AString[Pos('d',AString)] := 'e';
                end;
                ADatum := InternationalStrToFloat(AString);
                RealArray[SeriesIndex+InnerSeriesIndex] := ADatum;
                // Add the data point to the appropriate TLineSeries
                ASeries := SeriesList[SeriesIndex+InnerSeriesIndex];
                ASeries.AddXY(ATime, ADatum, '', clTeeColor);
              end;

              if FirstTime then
              begin
                DataString := TimeStep + Chr(9) + FloatToStr(ATime);
              end
              else
              begin
                DataString := Lines[DataIndex]
              end;
              for PointsIndex := SeriesIndex to SeriesIndex + ThisCount*rgSutra.Items.Count -1 do
              begin
                DataString := DataString + Chr(9) + FloatToStr(RealArray[PointsIndex]);
              end;
              if FirstTime then
              begin
                Lines.Add(DataString);
              end
              else
              begin
                Lines[DataIndex] := DataString;
              end;
            end;

            Inc(SeriesIndex,ThisCount*rgSutra.Items.Count);


            LineIndex := GetNextLine('NODE',LineIndex+1, SutraFile);
            MoreNodes := LineIndex > -1;
            FirstTime := False;
          end;




        end;
      end;





    finally
      begin
        // close the MODFLOW file

        for ChartIndex := 0 to Chart1.SeriesList.Count -1 do
        begin
          AChartSeries := Chart1.Series[ChartIndex];
          CellIndex := (ChartIndex div rgSutra.Items.Count) + 1;
          CorrectType := ((ChartIndex mod rgSutra.Items.Count) = rgSutra.ItemIndex);
          AChartSeries.Active := CorrectType
            and (dgDataNodes.Cells[1, CellIndex]
            = dgDataNodes.Columns[1].PickList.Strings[1]);
        end;

        RichEdit1.Lines := Lines;

      end;
    end;
  end;
  finally
    // enable or disable controls as appropriate.
    btnCancel.Enabled := False;
    btnRead.Enabled := True;
    adeCellCount.Enabled := True;
    dgDataNodes.Visible := True;
    dgDataPoints.Enabled := True;
    Lines.Free;
    SutraFile.Free;
    // restore the cursor
    Screen.Cursor := crDefault;
  end;
end;

procedure TfrmExtract.ReadNewSutraFile;
var
  SutraFile : TStringList; // the Sutra output file
  ADatum : double;  // a number read from the formatted head or drawdown file
  AString : string;
  RowIndex : integer; // indexes to the colums and rows
  PointsIndex : integer; // an index to the point for which a hydrograph is to be extracted.
  DataString : String;  // This is used to store the formatted data that will be placed on a line of the TMemo
  ATime : double; // elapsed time for the current time step
  CaptionString : string;
  ASeries : TLineSeries; // a hydrograph plotted on the chart
  RealArray : array of double; // an array of values read from
  RealArrayLength : integer;
  ChartIndex : integer;
  AChartSeries : TChartSeries;
  Lines : TStringlist;
  ALine : string;
  LineIndex : integer;
  CellIndex : integer;
  CorrectType : boolean;
  InnerSeriesIndex : integer;
  CaptionLine : integer;
  Captions : TStringList;
  ItemIndex : integer;
  Node, TimeStep : string;
  TypeIndex : integer;
  TitleLine : string;
  ObservationStart : integer;
//  MoreNodes : boolean;
//  DataIndex : integer;
begin
//  Put the name of the file on the caption of the form.
  Caption := 'Hydrograph Extractor: ' + OpenDialog1.FileName;
  // disable thing that shouldn't be changed while reading the file.
  btnSave.Enabled := True;
  btnCancel.Enabled := True;
  btnRead.Enabled := False;
  adeCellCount.Enabled := False;
  dgDataPoints.Enabled := False;
  SutraFile := TStringlist.Create;
  Lines := TStringlist.Create;
  // this may take a while so change the cursor.
  Screen.Cursor := crHourGlass;

  try
  begin
    // ClearSeriesList destroys all the TLineSeries on the chart and
    // clears SeriesList
    ClearSeriesList;

    // initialize CancelProcess. CancelProcess will be tested while the
    // file is being read to see if the user has pressed the cancel button.
    CancelProcess := False;

    // initialize RichEdit1
    RichEdit1.Clear;

    RowIndex := 0;
    SutraFile.LoadFromFile(OpenDialog1.FileName);
    try
      ObservationStart := GetNextLine(
        '## Time Step',
        0, SutraFile);

      if ObservationStart = -1 then
      begin
        MessageDlg(OpenDialog1.FileName + ' contains no observation nodes.',
          mtWarning, [mbOK], 0);
      end
      else
      begin
        Inc(ObservationStart);
        if not CancelProcess then
        begin
          Captions := TStringList.Create;
          try
            LineIndex := ObservationStart - 5;

            ALine := SutraFile[LineIndex];

            While (Length(ALine) > 0) do
            begin
              Inc(RowIndex);
              adeCellCount.Text := IntToStr(RowIndex);
              adeCellCountChange(adeCellCount);
              Node := GetStringBetween(ALine,'NODE','NODE');
              dgDataNodes.Cells[0,RowIndex] := Node;
              dgDataNodes.Cells[1,RowIndex] := dgDataNodes.Columns[1].PickList[0];
            end;

            CaptionLine := ObservationStart-1;

            ALine := SutraFile[CaptionLine];
            GetStringBetween(ALine,'Time (sec)','');
            While Pos(' ',ALine) <> 0 do
            begin
              AString := GetStringBetween(ALine,'',' ');
              if Captions.IndexOf(AString) = -1 then
              begin
                Captions.Add(AString);
              end;
            end;
            if (Captions.IndexOf(ALine) = -1) and (ALine <> '') then
            begin
              Captions.Add(ALine);
            end;

            ItemIndex := rgSutra.ItemIndex;
            rgSutra.Items := Captions;
            rgSutra.ItemIndex := ItemIndex;
          finally
            Captions.Free;
          end;

          RealArrayLength := StrToInt(frmExtract.adeCellCount.Text)
            *rgSutra.Items.Count;
          SetLength(RealArray, RealArrayLength);

          TitleLine := ' Time Step' + Chr(9) + 'Time (sec)';
          For RowIndex := 1 to dgDataNodes.RowCount -1 do
          begin
            for TypeIndex := 0 to rgSutra.Items.Count -1 do
            begin
              CaptionString := dgDataNodes.Cells[0,RowIndex];
              TitleLine := TitleLine + Chr(9) + rgSutra.Items[TypeIndex]
                + ' Node ' + CaptionString;

              ASeries := TLineSeries.Create(Chart1);
              ASeries.ParentChart := Chart1;
              SeriesList.Add(ASeries);
              ASeries.Title := CaptionString;
              ASeries.Pointer.Visible := True;
              ASeries.Active := cbPlot.Checked and (dgDataNodes.Cells[1, RowIndex]
                = dgDataNodes.Columns[1].PickList.Strings[1])
                and (rgSutra.ItemIndex = TypeIndex);

            end;
          end;

          Lines.Add(TitleLine);




          LineIndex := ObservationStart;

//          MoreNodes := LineIndex < SutraFile.Count ;
          While LineIndex < SutraFile.Count do
          begin


//            DataIndex := 0;
//            Inc(DataIndex);
            Application.ProcessMessages;
            if CancelProcess then
            begin
              break;
            end;
            ALine := Trim(SutraFile[LineIndex]);
            TimeStep := GetStringBetween(ALine,'',' ');
            AString := GetStringBetween(ALine,'',' ');
            if Pos('D',AString) > 0 then
            begin
              AString[Pos('D',AString)] := 'E';
            end;
            if Pos('d',AString) > 0 then
            begin
              AString[Pos('d',AString)] := 'e';
            end;
            InnerSeriesIndex := -1;
            ATime := InternationalStrToFloat(AString);
            while Length(ALine) >0 do
            begin
              Inc(InnerSeriesIndex);
              AString := GetStringBetween(ALine,'',' ');
              if Pos('D',AString) > 0 then
              begin
                AString[Pos('D',AString)] := 'E';
              end;
              if Pos('d',AString) > 0 then
              begin
                AString[Pos('d',AString)] := 'e';
              end;
              ADatum := InternationalStrToFloat(AString);
              RealArray[InnerSeriesIndex] := ADatum;
              // Add the data point to the appropriate TLineSeries
              ASeries := SeriesList[InnerSeriesIndex];
              ASeries.AddXY(ATime, ADatum, '', clTeeColor);
            end;

            DataString := TimeStep + Chr(9) + FloatToStr(ATime);

            for PointsIndex := 0 to InnerSeriesIndex do
            begin
              DataString := DataString + Chr(9) + FloatToStr(RealArray[PointsIndex]);
            end;
            Lines.Add(DataString);
//            Lines[DataIndex] := DataString;


            Inc(LineIndex);
//            MoreNodes := LineIndex < SutraFile.Count ;
          end;




        end;
      end;





    finally
      begin
        // close the SUTRA file

        for ChartIndex := 0 to Chart1.SeriesList.Count -1 do
        begin
          AChartSeries := Chart1.Series[ChartIndex];
          CellIndex := (ChartIndex div rgSutra.Items.Count) + 1;
          CorrectType := ((ChartIndex mod rgSutra.Items.Count) = rgSutra.ItemIndex);
          AChartSeries.Active := CorrectType
            and (dgDataNodes.Cells[1, CellIndex]
            = dgDataNodes.Columns[1].PickList.Strings[1]);
        end;

        RichEdit1.Lines := Lines;

      end;
    end;
  end;
  finally
    // enable or disable controls as appropriate.
    btnCancel.Enabled := False;
    btnRead.Enabled := True;
    adeCellCount.Enabled := True;
    dgDataNodes.Visible := True;
    dgDataPoints.Enabled := True;
    Lines.Free;
    SutraFile.Free;
    // restore the cursor
    Screen.Cursor := crDefault;
  end;
end;

function TfrmExtract.GetStringBetween(var AString : String; Before, After : String) : string;
var
  Position : integer;
begin
  result := AString;
  if Before <> '' then
  begin
    result := Copy(AString,
        Pos(Before,AString)+Length(Before),
        Length(AString));
    AString := result;
  end;

  if After <> '' then
  begin
    Position := Pos(After,result)-1;
    if Position = -1 then
    begin
      Position := Length(result);
    end;
    result := Copy(result,1,Position);
    AString := Copy(AString,Length(result)+1,Length(AString));
  end;

  result := Trim(result);
  AString := Trim(AString);
end;

function TfrmExtract.GetNextLine(SearchTerm : string; Start : integer;
  AStringList : TStringList) : integer;
var
  Index : integer;
begin
  result := -1;
  for Index := Start to AStringList.Count -1 do
  begin
    Application.ProcessMessages;
    if CancelProcess or (Pos(SearchTerm, AStringList.Strings[Index])> 0) then
    begin
      result := Index;
      break;
    end;
  end;
end;

procedure TfrmExtract.dgDataNodesSetEditText(Sender: TObject; ACol,
  ARow: Integer; const Value: String);
var
  ASeries : TChartSeries;
  ChartIndex : integer;
begin
  if (ACol = 1) and (ARow > 0) then
  begin
    ChartIndex := (ARow -1)*rgSutra.Items.Count + rgSutra.ItemIndex;
    if Chart1.SeriesList.Count > ChartIndex then
    begin
      ASeries := Chart1.Series[ChartIndex];
      ASeries.Active := dgDataNodes.Cells[1, ARow]
        = dgDataNodes.Columns[1].PickList.Strings[1];
    end;
  end;


end;

procedure TfrmExtract.rgSutraClick(Sender: TObject);
var
  ASeries : TChartSeries;
  ChartIndex, CellIndex : integer;
  CorrectType : boolean;
begin
  if rgProgramChoice.ItemIndex = 3 then
  begin
    Chart1.LeftAxis.Title.Caption := LowerCase(rgSutra.Items[rgSutra.ItemIndex]);
    for ChartIndex := 0 to Chart1.SeriesList.Count -1 do
    begin
      ASeries := Chart1.Series[ChartIndex];

      CellIndex := (ChartIndex div rgSutra.Items.Count) + 1;
      CorrectType := ((ChartIndex mod rgSutra.Items.Count) = rgSutra.ItemIndex);

      ASeries.Active := CorrectType
        and (dgDataNodes.Cells[1, CellIndex]
        = dgDataNodes.Columns[1].PickList.Strings[1]);

    end;
  end


end;

Initialization
GetDecimalChar;

end.
